import logging

from django.db import transaction
from django.db.models import PositiveIntegerField, BooleanField
from django.forms import model_to_dict

from isc_common import setAttr
from isc_common.auth.models.user import User
from isc_common.fields.related import ForeignKeyCascade
from isc_common.http.DSRequest import DSRequest
from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet
from isc_common.number import DelProps
from tracker.models.messages_theme import Messages_theme
from twits.models.chat_user_user import Chat_user_userManager

logger = logging.getLogger(__name__)


class Chat_user_userViewQuerySet(AuditQuerySet):
    def create(self, **kwargs):
        from twits.models.chat_user_user_theme import Chat_user_user_theme

        user_sender = User.objects.get(id=kwargs.get('user_sender_id'))
        user_reciver = User.objects.get(id=kwargs.get('user_reciver_id'))

        if user_sender.id == user_reciver.id:
            raise Exception(f'Чат с самим собою не допускается.')

        try:
            common = self.model.objects.get(user_sender=user_sender, user_reciver=user_reciver)
        except self.model.DoesNotExist:
            try:
                common = self.model.objects.get(user_sender=user_reciver, user_reciver=user_sender)
            except self.model.DoesNotExist:
                common = None

        if common:
            setAttr(kwargs, 'common_id', common.id)

        with transaction.atomic():
            theme, created = Messages_theme.objects.get_or_create(
                code=f'tetatet_{user_sender.get_short_name}_{user_reciver.get_short_name}',
                defaults=dict(
                    name=f'Чат: {user_sender.get_short_name} : {user_reciver.get_short_name}',
                    creator=User.admin_user(),
                    editing=False,
                    deliting=False,
                    parent=Messages_theme.tetatet_chats_theme(),
                    description='Группа создана автоматически'))

            chat_user_user = super().create(**kwargs)

            Chat_user_user_theme.objects.get_or_create(theme=theme, chat_user_user=chat_user_user)
            if not common:
                chat_user_user.common_id = chat_user_user.id
                chat_user_user.save()
        return chat_user_user

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Chat_user_userViewManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'common_id': record.common_id,
            'user_sender_id': record.user_sender.id,
            'user_reciver_id': record.user_reciver.id,
            'user_reciver__username': record.user_reciver.username,
            'user_reciver__first_name': record.user_reciver.first_name,
            'user_reciver__last_name': record.user_reciver.last_name,
            'user_reciver__email': record.user_reciver.email,
            'user_reciver__middle_name': record.user_reciver.middle_name,
            'user_reciver__last_login': record.user_reciver.last_login,
            'props': record.props,
            'compulsory_reading': record.props.compulsory_reading,
        }
        return DelProps(res)

    def get_queryset(self):
        return Chat_user_userViewQuerySet(self.model, using=self._db)

    def createFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        user_ids = data.get('user_ids', None)
        context_ids = data.get('context_ids', None)
        _data = []

        if user_ids and context_ids:
            if not isinstance(user_ids, list):
                user_ids = [user_ids]

            if not isinstance(context_ids, list):
                context_ids = [context_ids]

            for user_id in user_ids:
                for user_sender_id in context_ids:
                    res = super().create(user_reciver_id=user_id, user_sender_id=user_sender_id)
                    _data.append(model_to_dict(res))

        return data


class Chat_user_userView(AuditModel):
    user_sender = ForeignKeyCascade(User, related_name='user_sender_user_view')
    user_reciver = ForeignKeyCascade(User, related_name='user_reciver_user_view')
    common_id = PositiveIntegerField()
    props = Chat_user_userManager.props()

    objects = Chat_user_userViewManager()

    def __str__(self):
        return f'id: {self.id}, common_id: {self.common_id}, user_sender: [{self.user_sender}], user_reciver: [{self.user_reciver}]'

    class Meta:
        verbose_name = 'Чаты личной переписки'
        managed = False
        db_table = 'twits_chat_user_user_view'



import logging

from django.core.management import BaseCommand
from django.db import connection

from isc_common.models.mat_view import Mat_view

logger = logging.getLogger(__name__)


def refresh_mat_view(relname=None, schemaname=None, loger_func=lambda str: print(str)):
    def dbl_qutes_str(str):
        return f'"{str}"'

    if relname and schemaname:
        query = Mat_view.objects.filter(relname=relname, schemaname=schemaname)
    elif relname and not schemaname:
        query = Mat_view.objects.filter(relname=relname)
    elif not relname and schemaname:
        query = Mat_view.objects.filter(schemaname=schemaname)
    else:
        query = Mat_view.objects.filter()

    for mat_view in query:
        try:
            with connection.cursor() as cursor:
                # logger.debug(f'Creating: {m_view_name}')
                loger_func(f'Refreshing: {dbl_qutes_str(mat_view.relname)}')
                cursor.execute(f'REFRESH MATERIALIZED VIEW{dbl_qutes_str(mat_view.relname)};')
                loger_func(f'Refreshed')
                # cursor.execute(f'CREATE OR REPLACE FUNCTION tg_refresh_{mat_view.relname}() RETURNS trigger LANGUAGE plpgsql AS $$ BEGIN REFRESH MATERIALIZED VIEW {mat_view.relname}; RETURN NULL; END; $$;')
                # logger.debug(f'Created')
                # print(f'Function created')
        except Exception as ex:
            raise ex


class Command(BaseCommand):

    def handle(self, *args, **options):
        refresh_mat_view()



# Generated by Django 2.2.5 on 2019-09-09 13:57

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0062_auto_20190908_0610'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='messages',
            options={'verbose_name': 'Сообщения'},
        ),
    ]



import logging

from django.conf import settings
from django.core.management import BaseCommand

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    event = settings.EVENT_STACK.EVENTS_TEST

    def add_arguments(self, parser):
        # parser.add_argument('--mode', type=str)
        # parser.add_argument('--qty', type=int)
        ...

    def handle(self, *args, **options):
        self.event.send_message('Hello World')
        print('Done.')



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from tracker.models.message_user_2_user import Message_user_2_user, Message_user_2_userManager


@JsonResponseWithException()
def Message_user_2_user_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Message_user_2_user.objects.
                filter().
                get_range_rows1(
                request=request,
                function=Message_user_2_userManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Message_user_2_user_Add(request):
    return JsonResponse(DSResponseAdd(data=Message_user_2_user.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Message_user_2_user_Update(request):
    return JsonResponse(DSResponseUpdate(data=Message_user_2_user.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Message_user_2_user_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Message_user_2_user.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Message_user_2_user_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Message_user_2_user.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Message_user_2_user_Info(request):
    return JsonResponse(DSResponse(request=request, data=Message_user_2_user.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Message_user_2_user_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Message_user_2_user.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)



# Generated by Django 3.0.7 on 2020-06-12 16:25

from django.conf import settings
from django.db import migrations
import isc_common.fields.code_field


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('reports', '0007_jasper_reports_users_editor_identifier'),
    ]

    operations = [
        migrations.RunSQL('delete from reports_jasper_reports_users'),
        migrations.AlterField(
            model_name='jasper_reports_users',
            name='editor_identifier',
            field=isc_common.fields.code_field.CodeStrictField(),
        ),
        migrations.AlterUniqueTogether(
            name='jasper_reports_users',
            unique_together={('report', 'user', 'editor_identifier')},
        ),
    ]



import logging

from django.core.management import BaseCommand
from django.db import connection

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument('--filename', type=str)

    def handle(self, *args, **options):
        filename = options.get('filename')
        file = open(filename)

        with connection.cursor() as cursor:
            try:
                for sql_line in file.readline():
                    cursor.execute(sql_line)

            except Exception as e:
                logger.error(e)



import logging
from typing import List

from bitfield import Bit, BitHandler
from django.core.exceptions import EmptyResultSet
from django.db import connection, transaction
from django.db.models import Manager, QuerySet, Q, ForeignKey
from django.forms import model_to_dict

from isc_common import delAttr, setAttr
from isc_common.bit import TurnBitOn, TurnBitOff
from isc_common.fields.related import ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from isc_common.number import DelProps

logger = logging.getLogger(__name__)


class CommonQuerySet(QuerySet):

    def soft_restore(self):
        res = super().update(deleted_at=None)
        return res

    def hard_delete(self):
        res = super().delete()
        return res

    def alive(self):
        return self.filter(deleted_at=None)

    def dead(self):
        return self.exclude(deleted_at=None)

    def __init__(self, model=None, query=None, using=None, hints=None, alive_only=True, enabledAll=False):
        self.alive_only = alive_only
        self.enabledAll = enabledAll
        super().__init__(model=model, query=query, using=using, hints=hints)

    def get_field(self, field_name):
        fld = [x for x in self.model._meta.fields if x.name == field_name]
        if len(fld) > 0:
            return fld[0]
        return None

    def get_id_tuple(self, group):
        iter = [x for x in group]
        # print(iter)
        res = [x[1] for x in iter if x[0][1] == 'id']
        if len(res) > 0:
            return res[0]
        return None

    def getRecord(self, record):
        return record

    def getOperator(self, operator):
        if operator == "and":
            return Q.AND
        elif operator == "or":
            return Q.OR
        elif operator == "equals":
            return "exact"
        elif operator == "iEquals":
            return "iexact"
        elif operator == "notEqual":
            return "exact"
        elif operator == "iNotEqual":
            return "iexact"
        elif operator == "greaterThan":
            return "gt"
        elif operator == "lessThan":
            return "lt"
        elif operator == "greaterOrEqual":
            return "gte"
        elif operator == "lessOrEqual":
            return "lte"
        elif operator == "contains":
            return "contains"
        elif operator == "startsWith":
            return "startswith"
        elif operator == "endsWith":
            return "endswith"
        elif operator == "iContains":
            return "icontains"
        elif operator == "iStartsWith":
            return "istartswith"
        elif operator == "iEndsWith":
            return "iendswith"
        elif operator == "notContains":
            return "contains"
        elif operator == "notStartsWith":
            return "startswith"
        elif operator == "notEndsWith":
            return "endswith"
        elif operator == "iNotContains":
            return "icontains"
        elif operator == "iNotStartsWith":
            return "istartswith"
        elif operator == "iNotEndsWith":
            return "endswith"
        elif operator == "iBetween":
            return "range"
        elif operator == "iBetweenInclusive":
            return "range"
        elif operator == "matchesPattern":
            return "regex"
        elif operator == "iMatchesPattern":
            return "iregex"
        elif operator == "containsPattern":
            return "regex"
        elif operator == "startsWithPattern":
            return "regex"
        elif operator == "endsWithPattern":
            return "regex"
        elif operator == "iContainsPattern":
            return "iregex"
        elif operator == "iStartsWithPattern":
            return "iregex"
        elif operator == "iEndsWithPattern":
            return "iregex"
        elif operator == "regexp":
            return "regex"
        elif operator == "iregexp":
            return "iregex"
        elif operator == "isBlank":
            return "isblank"
        elif operator == "notBlank":
            return "notblank"
        elif operator == "isNull":
            return "isnull"
        elif operator == "notNull":
            return "notnull"
        elif operator == "inSet":
            return "in"
        elif operator == "notInSet":
            return "in"
        # elif operator == "equalsField":
        #     return ""
        # elif operator == "notEqualField":
        #     return ""
        # elif operator == "iEqualsField":
        #     return ""
        # elif operator == "iNotEqualField":
        #     return ""
        # elif operator == "greaterThanField":
        #     return ""
        # elif operator == "lessThanField":
        #     return ""
        # elif operator == "greaterOrEqualField":
        #     return ""
        # elif operator == "lessOrEqualField":
        #     return ""
        # elif operator == "containsField":
        #     return ""
        # elif operator == "startsWithField":
        #     return ""
        # elif operator == "endsWithField":
        #     return ""
        # elif operator == "iContainsField":
        #     return ""
        # elif operator == "iStartsWithField":
        #     return ""
        # elif operator == "iEndsWithField":
        #     return ""
        # elif operator == "notContainsField":
        #     return ""
        # elif operator == "notStartsWithField":
        #     return ""
        # elif operator == "notEndsWithField":
        #     return ""
        # elif operator == "iNotContainsField":
        #     return ""
        # elif operator == "iNotStartsWithField":
        #     return ""
        # elif operator == "iNotEndsWithField":
        #     return ""
        # elif operator == "not":
        #     return ""
        elif operator == "between":
            return "range"
        elif operator == "nbetweenInclusiveot":
            return "range"
        else:
            raise Exception(f'Неизветный operator: {operator}')

    def isNotOperator(self, operator):
        if operator == "notEqual":
            return True
        if operator == "iNotEqual":
            return True
        if operator == "notContains":
            return True
        if operator == "notStartsWith":
            return True
        if operator == "notEndsWith":
            return True
        if operator == "iNotContains":
            return True
        if operator == "iNotStartsWith":
            return True
        if operator == "iNotEndsWith":
            return True
        if operator == "notBlank":
            return True
        if operator == "notNull":
            return True
        if operator == "notInSet":
            return True
        if operator == "notEqualField":
            return True
        if operator == "iNotEqualField":
            return True
        if operator == "notContainsField":
            return True
        if operator == "notStartsWithField":
            return True
        if operator == "notEndsWithField":
            return True
        if operator == "iNotContainsField":
            return True
        if operator == "iNotStartsWithField":
            return True
        if operator == "iNotEndsWithField":
            return True
        if operator == "not":
            return True
        else:
            return False

    def textMatchStyleMapping(self, textMatchStyle):
        if textMatchStyle == 'exact':
            return 'iexact'
        elif textMatchStyle == 'exactCase':
            return 'exact'
        elif textMatchStyle == 'substring':
            return 'icontains'
        elif textMatchStyle == 'startsWith':
            return 'istartswith'
        else:
            return 'exact'
            # raise Exception(f'Неизветный textMatchStyle: {textMatchStyle}')

    def getValue(self, criterion):
        value = criterion.get('value')
        operator = criterion.get('operator')
        if value == None:
            if operator == "isNull":
                return True
            elif operator == "notNull":
                return True
            elif operator == "isBlank":
                return "''"
            elif operator == "notBlank":
                return "''"
            else:
                return value

        if isinstance(value, str):
            return value.strip()
        elif isinstance(value, int):
            return value
        elif isinstance(value, list):
            return value
        else:
            raise Exception(f'Неизветный value: {value}')

    def getCriteria(self, crireria, operator):
        res = Q()
        if isinstance(crireria, list):
            # <editor-fold desc="Fixed by Y.Andrew">
            criteria = [item for item in crireria if item.get('fieldName') != 'ts']
            # </editor-fold>

            for criterion in criteria:
                _criteria = criterion.get("criteria")
                if _criteria:
                    if self.isNotOperator(operator):
                        res.add(~Q(self.getCriteria(_criteria, criterion.get("operator"))), operator)
                    else:
                        res.add(Q(self.getCriteria(_criteria, criterion.get("operator"))), operator)
                elif criterion.get('fieldName') != 'ts':
                    _operator = self.getOperator(criterion.get('operator'))
                    if _operator == "notnull":
                        _operator = "isnull"

                    if _operator == 'isblank' or _operator == 'notblank':
                        _operator = ''
                    else:
                        _operator = f'__{_operator}'

                    if self.isNotOperator(criterion.get('operator')):
                        res.add(~Q(**{f"{criterion.get('fieldName')}{_operator}": self.getValue(criterion)}), operator)
                    else:
                        if _operator == '__icontains':
                            if self.getValue(criterion) != '':
                                res.add(Q(**{f"{criterion.get('fieldName')}{_operator}": self.getValue(criterion)}), operator)
                        else:
                            res.add(Q(**{f"{criterion.get('fieldName')}{_operator}": self.getValue(criterion)}), operator)

        return res

    def delete_underscore_element(self, data):
        if not isinstance(data, dict):
            raise Exception('data must be dict')

        res = dict()
        for key, value in data.items():
            if str(key).find('_', 0, 1) == -1:
                setAttr(res, key, value)
        return res

    def get_criteria(self, json=dict()):
        data = json.get('data', dict())
        _data = dict()
        if data != None:
            for key, value in data.items():
                if value != None and key != 'full_name':
                    setAttr(_data, key, value)
        data = _data
        delAttr(data, 'ts')

        # fields_name = [field.name for field in self.model._meta.fields]
        _data = data.copy()
        delAttr(_data, 'guid')
        for key, value in data.items():
            if key.find('__id') == -1 and key.find('_id') != -1:
                if len([item.name for item in self.model._meta.fields if item.name == key]) == 0:
                    delAttr(_data, key)
                    setAttr(_data, key.replace('_id', '__id'), value)
            elif isinstance(value, str) and value.strip() == '':
                delAttr(_data, key)

        data = _data

        criteria = Q()
        operator = Q.AND

        if data.get('_constructor') == 'AdvancedCriteria':
            operator = self.getOperator(data.get("operator"))
            _criteria = data.get("criteria")
            if self.isNotOperator(data.get("operator")):
                criteria.add(~Q(self.getCriteria(_criteria, operator)), operator)
            else:
                criteria.add(Q(self.getCriteria(_criteria, operator)), operator)
        else:
            # "exact"	case-insensitive exact match ("foo" matches "foo" and "FoO", but not "FooBar")  iexact
            # "exactCase"	case-sensitive exact match ("foo" matches only "foo")                       exact
            # "substring"	case-insenstive substring match ("foo" matches "foobar" and "BarFoo")       icontains
            # "startsWith"	case-insensitive prefix match ("foo" matches "FooBar" but not "BarFoo")     istartswith
            textMatchStyle = self.textMatchStyleMapping(json.get('textMatchStyle'))
            for key, value in data.items():
                if isinstance(value, Bit):
                    criteria.add(Q(**{f"{key}": value}), operator)
                elif isinstance(value, list):
                    criteria.add(Q(**{f"{key}__in": value}), operator)
                else:
                    if textMatchStyle == "iexact" and isinstance(value, int):
                        textMatchStyle = "exact"

                    if isinstance(value, int) or isinstance(value, bool):
                        criteria.add(Q(**{f"{key}": value}), operator)
                    elif value == 'null':
                        criteria.add(Q(**{f"{key}__isnull": True}), operator)
                    else:
                        criteria.add(Q(**{f"{key}__{textMatchStyle}": value}), operator)
        return criteria

    def _get_range_rows(self, *args, **kwargs):
        json = kwargs.get('json')
        distinct_field_names = kwargs.get('distinct_field_names')
        end = kwargs.get('end')
        start = kwargs.get('start')

        criteria = self.get_criteria(json=json)

        ob = json.get("sortBy", [])

        if isinstance(distinct_field_names, tuple) or (isinstance(distinct_field_names, list)):
            if self.alive_only == False:
                if start != None and end != None:
                    queryResult = super().filter(*args, criteria).distinct(*distinct_field_names)[start:end]
                elif start != None and end == None:
                    queryResult = super().filter(*args, criteria).distinct(*distinct_field_names)[start:]
                elif end != None and start == None:
                    queryResult = super().filter(*args, criteria).distinct(*distinct_field_names)[:end]
                else:
                    queryResult = super().filter(*args, criteria).distinct(*distinct_field_names)
            else:
                if start != None and end != None:
                    queryResult = super().filter(*args, criteria).filter(deleted_at__isnull=True).distinct(*distinct_field_names)[start:end]
                elif start != None and end == None:
                    queryResult = super().filter(*args, criteria).filter(deleted_at__isnull=True).distinct(*distinct_field_names)[start:]
                elif end != None and start == None:
                    queryResult = super().filter(*args, criteria).filter(deleted_at__isnull=True).distinct(*distinct_field_names)[:end]
                else:
                    queryResult = super().filter(*args, criteria).filter(deleted_at__isnull=True).distinct(*distinct_field_names)
        else:
            if self.alive_only == False:
                if start != None and end != None:
                    queryResult = super().order_by(*ob).filter(*args, criteria).distinct()[start:end]
                elif start != None and end == None:
                    queryResult = super().order_by(*ob).filter(*args, criteria)[start:]
                elif end != None and start == None:
                    queryResult = super().order_by(*ob).filter(*args, criteria)[:end]
                else:
                    queryResult = super().order_by(*ob).filter(*args, criteria)
            else:
                if start != None and end != None:
                    queryResult = super().order_by(*ob).filter(*args, criteria).filter(deleted_at__isnull=True)[start:end]
                elif start != None and end == None:
                    queryResult = super().order_by(*ob).filter(*args, criteria).filter(deleted_at__isnull=True)[start:]
                elif end != None and start == None:
                    queryResult = super().order_by(*ob).filter(*args, criteria).filter(deleted_at__isnull=True)[:end]
                else:
                    queryResult = super().order_by(*ob).filter(*args, criteria).filter(deleted_at__isnull=True)
        return queryResult

    def get_range_rows(self, start=None, end=None, function=None, json=None, distinct_field_names=None, user=None, *args, **kwargs):
        queryResult = self._get_range_rows(*args, start=start, end=end, function=function, json=json, distinct_field_names=distinct_field_names)

        try:
            logger.debug(f'\n\n{queryResult.query}\n')
        except EmptyResultSet:
            pass

        if function:
            res = [function(record) for record in queryResult]
            return res
        else:
            res = [model_to_dict(record) for record in queryResult]
            return res

    def get_range_rows1(self, request, function=None, distinct_field_names=None, remove_fields=None):
        request = DSRequest(request=request)
        if isinstance(remove_fields, list):
            data = request.json.get('data')
            deleted = False
            for remove_field in remove_fields:
                if delAttr(data, remove_field) == True:
                    deleted = True
            if deleted == True:
                setAttr(request.json, 'data', data)

        self.alive_only = request.alive_only
        self.enabledAll = request.enabledAll
        res = self.get_range_rows(
            start=request.startRow,
            end=request.endRow,
            function=function,
            distinct_field_names=distinct_field_names,
            json=request.json,
            criteria=request.get_criteria(),
            user=request.user
        )
        return res

    def get_range_rows2(self, *args, **kwargs):
        function = kwargs.get('function')
        user_id = kwargs.get('user_id')
        username = kwargs.get('username')
        ws_channel = kwargs.get('ws_channel')
        ws_port = kwargs.get('ws_port')
        host = kwargs.get('host')
        queryResult = self._get_range_rows(*args, **kwargs)

        try:
            logger.debug(f'\n\n{queryResult.query}\n')
        except Exception as ex:
            logger.warning(ex)

        if function:
            res = [function(record=record, user_id=user_id, ws_channel=ws_channel, ws_port=ws_port, host=host, username=username) for record in queryResult]
            return res
        else:
            res = [model_to_dict(record) for record in queryResult]
            return res

    def get_range_rows3(self, start=None, end=None, function=None, json=None, distinct_field_names=None, user=None, *args, **kwargs):
        queryResult = self._get_range_rows(*args, start=start, end=end, function=function, json=json, distinct_field_names=distinct_field_names)

        logger.debug(f'\n\n{queryResult.query}\n')
        if function:
            res = [function(record, self.enabledAll) for record in queryResult]
            return res
        else:
            res = [model_to_dict(record) for record in queryResult]
            return res

    def get_range_rows11(self, request, **kwargs):
        distinct_field_names = kwargs.get('distinct_field_names')
        function = kwargs.get('function')

        request = DSRequest(request=request)
        self.alive_only = request.alive_only
        self.enabledAll = request.enabledAll
        return self.get_range_rows2(
            start=request.startRow,
            end=request.endRow,
            function=function,
            distinct_field_names=distinct_field_names,
            json=request.json,
            username=request.username,
            user_id=request.user_id,
            ws_channel=request.ws_channel,
            ws_port=request.ws_port,
            host=request.host,
        )

    def get_info(self, request, *args):
        request = DSRequest(request=request)
        delAttr(request.json.get('data'), 'full_name')
        criteria = self.get_criteria(json=request.json)
        cnt = super().filter(*args, criteria).count()
        cnt_all = super().filter().count()
        return dict(qty_rows=cnt, all_rows=cnt_all)


class CommonManager(Manager):
    @staticmethod
    def delete_underscore_element(data):
        if not isinstance(data, dict):
            raise Exception('data must be dict')

        res = dict()
        for key, value in data.items():
            if str(key).find('_', 0, 1) == -1:
                setAttr(res, key, value)
        return res

    def delete_dbl_underscore_element(self, data):
        if not isinstance(data, dict):
            raise Exception('data must be dict')

        res = dict()
        for key, value in data.items():
            if str(key).find('_', 0, 1) == -1 and str(key).find('__') == -1:
                setAttr(res, key, value)
        return res

    def refresh_mat_view(self, mat_view):
        with connection.cursor() as cursor:
            logger.debug(f'Refreshing: {mat_view}')
            cursor.execute(f'REFRESH MATERIALIZED VIEW CONCURRENTLY {mat_view};')
            logger.debug(f'Refreshed')

    def get_field(self, field_name, model=None):
        if not model:
            model = self.model

        fld = [x for x in model._meta.fields if x.name == field_name]
        if fld and len(fld) > 0:
            return fld

        if field_name.rfind('_id') + 3 == len(field_name):
            fld = [x for x in model._meta.fields if x.name == field_name.replace('_id', '')]
            if fld and len(fld) > 0:
                return fld

        list_foreign = field_name.split('__')
        if len(list_foreign) > 1:
            list_foreign, spliter = [x.split('_') for x in list_foreign], '__'
        else:
            list_foreign, spliter = field_name.split('_'), '_'

        if len(list_foreign) > 1:
            if isinstance(list_foreign, list):
                for _list_foreign in list_foreign:
                    if isinstance(_list_foreign, list):
                        fld = self.get_field(spliter.join(_list_foreign))
                    else:
                        fld = [x for x in model._meta.fields if x.name == _list_foreign]
                        if len(fld) == 1 and isinstance(fld[0], ForeignKeyProtect):
                            fld = self.get_field(spliter.join(list_foreign[1:]), model=fld[0].related_model)
                        else:
                            break
        else:
            fld = [x for x in model._meta.fields if x.name == field_name]
        if fld and len(fld) > 0:
            return fld

        return None

    def get_queryset(self):
        return CommonQuerySet(self.model, using=self._db)

    def get_range_rows1(self, request, function=None, distinct_field_names=None):
        request = DSRequest(request=request)
        self.alive_only = request.alive_only
        self.enabledAll = request.enabledAll
        return self.get_queryset().get_range_rows(start=request.startRow, end=request.endRow, function=function, distinct_field_names=distinct_field_names, json=request.json)

    def createFromRequest(self, request, removed=None, propsArr: List = None):
        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()

        if _data.get('parent') != None and isinstance(_data.get('parent'), int):
            setAttr(_data, 'parent_id', _data.get('parent'))
            delAttr(_data, 'parent')

        delAttr(_data, 'dataSource')
        delAttr(_data, 'operationType')
        delAttr(_data, 'textMatchStyle')
        delAttr(_data, 'form')
        self._remove_prop(_data, removed)

        props = self.get_prp(data=_data, propsArr=propsArr)
        if props > -1:
            setAttr(_data, 'props', props)

        res = super().create(**_data)
        try:
            full_name = res.full_name
        except:
            full_name = None
        res = model_to_dict(res)
        if full_name:
            setAttr(res, 'full_name', full_name)

        setAttr(res, 'isFolder', False)
        data.update(DelProps(res))
        return data

    def _dataIsArray(self, data):
        return isinstance(data, list)

    def _remove_prop(self, data, removed):
        if isinstance(data, dict) and isinstance(removed, list):
            for removed_item in removed:
                delAttr(data, removed_item)

    def _remove_prop_(self, data):
        res = None
        if isinstance(data, dict):
            res = dict()
            for key in data.keys():
                if not key.startswith('_'):
                    setAttr(res, key, data.get(key))
        return res

    def clone_data(self, data):
        _data = dict()
        if isinstance(data, dict):
            for key, val in data.items():
                if self.get_field(key):
                    setAttr(_data, key, val)
        return _data

    def get_prp(self, data, propsArr, props_name='props'):
        if isinstance(data, dict):
            props = data.get(props_name, 0)
            if isinstance(propsArr, list):
                idx = 0
                for prop in propsArr:
                    _prop = data.get(prop)

                    if _prop == True:
                        props = TurnBitOn(props, idx)
                    else:
                        props = TurnBitOff(props, idx)

                    delAttr(data, prop)
                    idx += 1
                return props
            else:
                return -1
        return -1

    def updateFromRequest(self, request, removed=None, function=None, propsArr: List = None):
        if not isinstance(request, DSRequest):
            request = DSRequest(request=request)
        data = request.get_data()

        _data = data.copy()

        if self._dataIsArray(data):
            data = list(data.values())
            for data_item in data:
                self._remove_prop(data_item, removed)
                data_item = self._remove_prop_(data_item)
                if isinstance(data_item, dict):
                    if function:
                        function(data, data_item)
                    else:
                        props = self.get_prp(data=data_item, propsArr=propsArr)
                        cloned_data = self.clone_data(data)
                        if props > -1:
                            setAttr(cloned_data, 'props', props)

                        if data_item.get('id'):
                            super().update_or_create(id=data_item.get('id'), defaults=cloned_data)
                        else:
                            super().create(**cloned_data)
        else:
            self._remove_prop(data, removed)
            data = self._remove_prop_(data)
            if function:
                function(data)
            else:
                props = self.get_prp(data=data, propsArr=propsArr)
                cloned_data = self.clone_data(data)
                if props > -1:
                    setAttr(cloned_data, 'props', props)
                if request.get_id():
                    delAttr(cloned_data, 'id')
                    parent = cloned_data.get('parent')
                    if isinstance(parent, int):
                        delAttr(cloned_data, 'parent')
                        setAttr(cloned_data, 'parent_id', parent)
                    super().update_or_create(id=request.get_id(), defaults=cloned_data)
                else:
                    new_item = super().create(**cloned_data)
                    setAttr(_data, 'id', new_item.id)
        return _data

    def copyFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        old = self.get(id=data.get('id'))
        _old = model_to_dict(old)
        delAttr(_old, 'id')
        _old1 = dict()

        for key, value in _old.items():
            if isinstance(self.get_field(key), ForeignKey):
                setAttr(_old1, f'{key}_id', value)
            else:
                setAttr(_old1, key, value)

        _old = _old1

        for field_name in [field.name for field in old._meta.fields if field._unique == True]:
            num = 1
            value = None
            while True:
                try:
                    value = _old.get(field_name)
                    if isinstance(value, str):
                        value = f'{_old.get(field_name)}-{num}'
                        eval(f'self.get({field_name}="{value}")', dict(self=self))
                    else:
                        break
                    num += 1
                except old.DoesNotExist:
                    setAttr(_old, field_name, value)
                    break

        for unique_together in old._meta.unique_together:
            num = 1
            get_dict = dict()
            while True:
                try:
                    get_str = ""
                    for unique_together_item in unique_together:
                        value = _old.get(f'{unique_together_item}_id')
                        if isinstance(self.get_field(unique_together_item), ForeignKey):
                            get_str += f'{unique_together_item}_id={value},'
                            setAttr(_old, f'{unique_together_item}_id', value)
                            delAttr(_old, unique_together_item)
                        elif isinstance(value, int):
                            value += 1
                            get_str += f'{unique_together_item}={value},'
                            setAttr(dict, unique_together_item, value)
                        else:
                            value = f'{_old.get(unique_together_item)}-{num}'
                            get_str += f'{unique_together_item}="{value}",'
                            setAttr(get_dict, unique_together_item, value)

                    eval(f'self.get({get_str})', dict(self=self))
                    num += 1
                except old.DoesNotExist:
                    for key, val in get_dict.items():
                        setAttr(_old, key, val)
                    break

        kwargs = dict()
        for key, val in _old.items():
            # if key.find('_id') != -1:
            field = self.get_field(key)
            if isinstance(field, list):
                field = field[0]
            if field != None:
                if isinstance(field, ForeignKey):
                    setAttr(kwargs, f'{key}_id', val)
                else:
                    setAttr(kwargs, key, val)
            # else:
            #     setAttr(kwargs, key, val)

        new = self.create(**kwargs)
        setAttr(_old, 'id', new.id)
        return _old

    def lookupFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()

        _data = dict()
        _key = None

        for key, value in data.items():
            if key.find('__') != -1:
                if value == 'null':
                    setAttr(_data, key[key.find('__') + 2:], None)
                else:
                    setAttr(_data, key[key.find('__') + 2:], value)
                _key = key[: key.find('__')]
                # break

        field = self.get_field(_key)
        res = None
        if isinstance(field, list):
            field = field[0]

        if field:
            # if field.null:
            if field == None:
                setAttr(_data, 'id', None)
                res = _data
            else:
                _data = {k: v for k, v in _data.items() if v != None}
                if isinstance(_data, dict) and len(_data) > 0:
                    try:
                        res = field.related_model.objects.filter(**_data)
                        if len(res) >= 1:
                            res = res[0]
                            res = model_to_dict(res)
                            for key, value in data.items():
                                if str(key).find('full_name') != -1:
                                    setAttr(res, 'full_name', value)
                    except Exception as ex:
                        logger.error(ex)
                else:
                    logger.warning(f'Element {_data} not found')

        _res = {}
        if isinstance(res, dict):
            for key, value in res.items():
                if not isinstance(value, list) and not isinstance(value, dict):
                    if isinstance(value, BitHandler):
                        props = res.get('props')._value
                        setAttr(_res, 'props', props)
                    else:
                        setAttr(_res, key, value)
        return _res

    def deleteFromRequest(self, request, removed=None, ):
        request = DSRequest(request=request)
        res = 0
        tuple_ids = request.get_olds_tuple_ids()
        with transaction.atomic():
            for id, mode in tuple_ids:
                if mode == 'hide':
                    super().filter(id=id).soft_delete()
                    res += 1
                elif mode == 'visible':
                    super().filter(id=id).soft_restore()
                else:
                    qty, _ = super().filter(id=id).delete()
                    res += qty
        return res

    def get_ex(self, *args, **kwargs):
        """
        Perform the query and return a single object matching the given
        keyword arguments.
        """
        clone = super().get_queryset().filter(*args, **kwargs)
        num = len(clone)
        if num >= 1:
            return clone._result_cache
        if not num:
            return None
        return None



# Generated by Django 2.1.7 on 2019-03-07 00:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0010_auto_20190306_1418'),
    ]

    operations = [
        migrations.AlterField(
            model_name='params',
            name='key',
            field=models.TextField(),
        ),
    ]



import json
import logging

import websocket

logger = logging.getLogger(__name__)


class WebSocketExt(websocket.WebSocket):
    def sendEx(self, message):
        if not isinstance(message, dict):
            raise Exception(f'Message must be a dict.')
        self.send(payload=json.dumps(message))


class WebSocketAppExt(websocket.WebSocketApp):
    def _send(self, message):
        if not isinstance(message, dict):
            raise Exception(f'Message must be a dict.')
        self.send(data=json.dumps(message))

    def show(self, title, id, label_contents=None, cntAll=0, caption=None):
        self._send(dict(
            caption=caption,
            cntAll=cntAll,
            labelContents=label_contents,
            progressBarId=id,
            title=title,
            type='show_progress',
        ))

    def setContentsLabel(self, labelContents, id):
        self._send(dict(type='set_contents_label', labelContents=labelContents, progressBarId=id))

    def setTitleProgress(self, title, id):
        self._send(dict(type='set_title_progress', title=title, progressBarId=id))

    def setPercentsDone(self, percent, id):
        self._send(dict(type='set_percent_done_progress', percent=percent, progressBarId=id))

    def setCntDone(self, cnt, id):
        self._send(dict(type='set_cnt_done_progress', cnt=cnt, progressBarId=id))

    def closed(self, id):
        self._send(dict(type='close_progress', progressBarId=id))


class ProgressStack:
    host = None
    port = None
    channel = None
    user_id = None
    props = False

    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v() if callable(v) else v)

        if not self.host:
            raise Exception(f'Not specified a host.')

        if not self.port:
            raise Exception(f'Not specified a port.')

        if not self.channel:
            raise Exception(f'Not specified a channel.')

        self.url = f'ws://{self.host}:{self.port}/ws/{self.channel}/'

    def _send(self, message):
        if not isinstance(message, dict):
            raise Exception(f'dict: {dict} Message must be a dict.')

        # logger.debug(f'url={self.url}')
        self.ws = websocket.create_connection(url=self.url, class_=WebSocketExt)
        self.ws.sendEx(message)
        self.ws.close()

    def show(self, title, id, label_contents=None, cntAll=0, caption=None):
        if self.user_id == None:
            raise Exception(f'user_id should be no None')
        if id == None:
            raise Exception(f'user_id should be no None')

        self._send(dict(
            caption=caption,
            channel=self.channel,
            cntAll=cntAll,
            labelContents=label_contents,
            progressBarId=id,
            props=self.props,
            title=title,
            type='show_progress',
            user_id=self.user_id
        ))

    def close(self, id):
        self._send(dict(type='close_progress', progressBarId=id))

    def setContentsLabel(self, labelContents, id):
        self._send(dict(type='set_contents_label', labelContents=labelContents, progressBarId=id))

    def setCntAll(self, cntAll, id):
        self._send(dict(type='set_cnt_all', cntAll=cntAll, progressBarId=id))

    def setTitleProgress(self, title, id):
        self._send(dict(type='set_title_progress', title=title, progressBarId=id))

    def setPercentsDone(self, percent, id):
        self._send(dict(type='set_percent_done_progress', percent=percent, progressBarId=id))

    def setCntDone(self, cnt, id):
        self._send(dict(type='set_cnt_done_progress', cnt=cnt, progressBarId=id))

    def sendInfo(self, message):
        self._send(dict(type='info', message=message))

    def sendWarn(self, message):
        self._send(dict(type='warn', message=message))

    def sendError(self, message, stackTrace=None):
        self._send(dict(type='error', message=message, stackTrace=stackTrace))

    def sendMessage(self, type, message):
        self._send(dict(type=type, message=message))


class ProgressSerialStack:
    host = None
    port = None
    channel = None

    def __init__(self, func, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v() if callable(v) else v)

        if not self.host:
            raise Exception(f'Not specified a host.')

        if not self.port:
            raise Exception(f'Not specified a port.')

        if not self.channel:
            raise Exception(f'Not specified a channel.')

        url = f'ws://{self.host}:{self.port}/ws/{self.channel}/'

        def on_open(ws):
            try:
                import thread
            except ImportError:
                import _thread as thread

            def run(*args, **kwargs):
                try:
                    func(ws, **kwargs)
                    ws.close()
                except:
                    ws.close()

            thread.start_new_thread(run, (), kwargs)

        ws = WebSocketAppExt(url)
        ws.on_open = on_open
        ws.run_forever()



import logging


class Logger(logging.Logger):
    def logging(self, message, level='info'):
        if level == 'debug':
            self.debug(message)
        elif level == 'warning':
            self.warning(message)
        elif level == 'info':
            self.info(message)
        elif level == 'error':
            self.error(message)



# Generated by Django 2.2.3 on 2019-07-06 22:14

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('one_c', '0014_auto_20190706_2103'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Uuid_2_id',
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-30 22:50

from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('twits', '0003_auto_20190822_2250'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='Chat_user_permission',
            new_name='Chat_user_user',
        ),
        migrations.DeleteModel(
            name='Chat_permission',
        ),
    ]



from django.urls import path

from isc_common.views import standard_colors

urlpatterns = [

    path('Standard_colors/Fetch/', standard_colors.Standard_colors_Fetch),
    path('Standard_colors/Add', standard_colors.Standard_colors_Add),
    path('Standard_colors/Update', standard_colors.Standard_colors_Update),
    path('Standard_colors/Remove', standard_colors.Standard_colors_Remove),
    path('Standard_colors/Lookup/', standard_colors.Standard_colors_Lookup),
    path('Standard_colors/Info/', standard_colors.Standard_colors_Info),
    path('Standard_colors/Copy', standard_colors.Standard_colors_Copy),

]



# python
import base64
import os

# third party
from django.core.files.base import ContentFile
from django.db.models import BinaryField
from django.utils.deconstruct import deconstructible
from django.utils.http import urlencode

# project
from isc_common import delete_drive_leter
from isc_common.storages.storage_ex import StorageEx


class NameException(Exception):
    pass


@deconstructible
class DatabaseFileStorage(StorageEx):
    """File storage system that saves models' FileFields in the database.
    Intended for use with Models' FileFields.
    Uses a specific model for each FileField of each Model.
    """

    NAME_FORMAT_HINT = '<app>.<model>/<content_field>/<filename_field>/<mimetype_field>/<filename>'
    NAME_FORMAT_HINT1 = '<app>.<model>/<content_field>/<filename_field>/<mimetype_field>'

    def _get_encoded_bytes_from_file(self, content_field, _file):
        _file.seek(0)
        file_content = _file.read()
        encoded = base64.b64encode(file_content)
        if isinstance(content_field, BinaryField):
            return encoded
        return encoded.decode('utf-8')

    def _get_file_from_encoded_bytes(self, encoded_bytes):
        file_buffer = base64.b64decode(encoded_bytes)
        return ContentFile(file_buffer)

    def _get_storage_attributes(self, name):
        try:
            (
                model_class_path,
                content_field,
                filename_field,
                mimetype_field,
                filename
            ) = name.split(os.sep, 4)
        except ValueError:
            raise NameException(
                'Wrong name format. Got {} ; should be {}'.format(name, self.NAME_FORMAT_HINT))

        filename = delete_drive_leter(filename)

        return {
            'model_class_path': model_class_path,
            'content_field': content_field,
            'filename_field': filename_field,
            'mimetype_field': mimetype_field,
            'filename': filename,
        }

    def _get_storage_attributes1(self, name):
        try:
            (
                model_class_path,
                content_field,
                filename_field,
                mimetype_field,
            ) = name.split(os.sep, 4)
        except ValueError:
            raise NameException(
                'Wrong name format. Got {} ; should be {}'.format(name, self.NAME_FORMAT_HINT1))

        return {
            'model_class_path': model_class_path,
            'content_field': content_field,
            'filename_field': filename_field,
            'mimetype_field': mimetype_field,
        }

    def _open(self, name, mode='rb'):
        assert mode[0] in 'rwab'

        if os.sep != '/':
            self.download_from = self.download_from.replace('/', os.sep)

        storage_attrs = self._get_storage_attributes1(self.download_from)
        model_class_path = storage_attrs['model_class_path']
        content_field = storage_attrs['content_field']
        filename_field = storage_attrs['filename_field']
        mimetype_field = storage_attrs['mimetype_field']
        filename = name

        model_cls = self._get_model_cls(model_class_path)
        model_instance = model_cls.objects.only(content_field, mimetype_field).get(**{filename_field: name})
        encoded_bytes = getattr(model_instance, content_field)

        _file = self._get_file_from_encoded_bytes(encoded_bytes)
        _file.filename = filename
        _file.mimetype = getattr(model_instance, mimetype_field)
        return _file

    def _save(self, name, content):
        storage_attrs = self._get_storage_attributes(name)
        model_class_path = storage_attrs['model_class_path']
        content_field_name = storage_attrs['content_field']
        filename_field_name = storage_attrs['filename_field']
        mimetype_field_name = storage_attrs['mimetype_field']

        model_cls = self._get_model_cls(model_class_path)
        new_filename = delete_drive_leter(content.file.name)

        content_field = model_cls._meta.get_field(content_field_name)
        encoded_bytes = self._get_encoded_bytes_from_file(content_field, content)

        mimetype = (
                getattr(content, 'content_type', None) or  # Django >= 1.11
                getattr(content.file, 'content_type', None) or  # Django < 1.11
                'application/octet-stream'  # Fallback
        )

        model_cls.objects.create(**{
            content_field_name: encoded_bytes,
            filename_field_name: new_filename,
            mimetype_field_name: mimetype,
        })
        return new_filename

    def delete(self, name):
        if os.sep != '/':  # Windows fix (see a6d4707) # pragma: no cover
            name = name.replace('/', os.sep)
        storage_attrs = self._get_storage_attributes(name)
        model_class_path = storage_attrs['model_class_path']
        filename_field = storage_attrs['filename_field']

        model_cls = self._get_model_cls(model_class_path)
        model_cls.objects.filter(**{filename_field: name}).delete()

    def exists(self, name):
        if os.sep != '/':  # Windows fix (see a6d4707) # pragma: no cover
            name = name.replace('/', os.sep)
        try:
            storage_attrs = self._get_storage_attributes(name)

        except NameException:
            return False

        model_class_path = storage_attrs['model_class_path']
        filename_field = storage_attrs['filename_field']

        model_cls = self._get_model_cls(model_class_path)
        return model_cls.objects.filter(**{filename_field: name}).exists()

    def url(self, name):
        return urlencode({'name': name})



from django.db import connection

from isc_common.auth.models.user import User
from isc_common.auth.models.user_permission import User_permission
from isc_common.auth.models.widgets_trees import Widgets_trees
from isc_common.http.DSRequest import DSRequest
from isc_common.http.RPCResponse import RPCResponseConstant


class GetPermissionRequest(DSRequest):
    def __init__(self, request):
        DSRequest.__init__(self, request)
        data = self.get_data()
        id = data.get('id', None)
        res = []
        widget = None

        try:
            id_widget = data.get('id_widget', None)
            widget = Widgets_trees.objects.get(id_widget=id_widget)
        except Widgets_trees.DoesNotExist:
            pass

        if widget:
            user = User.objects.get(id=id)
            user_permission = None
            try:
                user_permission = User_permission.objects.get(user=user, widget=widget).permission
            except User_permission.DoesNotExist:
                pass

            grouops_first_level = list(user.usergroup.filter(parent__isnull=True).order_by('id'))
            if len(grouops_first_level) == 0:
                grouops_first_level = list(user.usergroup.filter())


            if user_permission:
                res = [dict(user_id=user.id, permission=user_permission)]


            with connection.cursor() as cursor:
                for group in grouops_first_level:
                    cursor.execute("""WITH RECURSIVE r AS (
                                       SELECT id group_id, 1 AS level
                                       FROM isc_common_usergroup
                                       WHERE id = %s
                                    
                                       UNION ALL
                                    
                                       SELECT g.id  group_id, r.level + 1 AS level
                                       FROM isc_common_usergroup g
                                         JOIN r
                                              ON (g.parent_id = r.group_id) 
                                    )
                                    
                                    SELECT gp.usergroup_id, r.level, gp.permission FROM r 
                                        join isc_common_user_usergroup ug 
                                            on (r.group_id = ug.usergroup_id)
                                        join isc_common_usergroup_permission gp
                                            on (gp.usergroup_id = ug.usergroup_id)
                                        join isc_common_widgets_trees wt
                                            on (wt.id = gp.widget_id)
                                            
                                        where ug.user_id = %s
                                        and wt.id_widget = %s
                                        order by r.level desc """, [group.id, user.id, id_widget])
                    row = cursor.fetchall()
                    for p in row:
                        group = p[0]
                        level = p[1]
                        permission = p[2]
                        res.append(dict(level=level, permission=permission))

                        # print(f'group : {group}, level: {level}, permission: {permission}')

        self.response = dict(response=dict(status=RPCResponseConstant.statusSuccess), data=res)



# Generated by Django 2.2.5 on 2019-09-04 17:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0053_auto_20190904_1757'),
    ]

    operations = [
        migrations.AlterField(
            model_name='messages',
            name='checksum',
            field=models.CharField(default=None, max_length=255),
        ),
    ]



# Generated by Django 2.2.2 on 2019-06-10 15:55

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('one_c', '0006_auto_20190610_1533'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='documents_param_1c',
            name='document',
        ),
    ]



from code_generator.app.model import Model


class ModelCodeBlock(Model):
    code = None

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        if self.code == None:
            raise Exception(f'Не задан код.')

    def __str__(self):
        return self.code.split('\n')


class TypeScriptCodeBlock(ModelCodeBlock):
    pass


class PythonCodeBlock(ModelCodeBlock):
    pass



import logging

from isc_common.auth.models.user import User
from isc_common.fields.related import ForeignKeyProtect
from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet

logger = logging.getLogger(__name__)


class AuditHistoryQuerySet(AuditQuerySet):
    pass


class AuditHistoryManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'code': record.code,
            'name': record.name,
            'description': record.description,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return AuditHistoryQuerySet(self.model, using=self._db)


class AuditHistory(AuditModel):
    hcreator = ForeignKeyProtect(User)

    objects = AuditHistoryManager()

    def __str__(self):
        return f'ID:{self.id}'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'История изменений'
        abstract = True



from isc_common.http.DSRequest import DSRequest
from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from tracker.models.messages import Messages
from tracker.models.messages_admin_view import Messages_admin_view, Messages_admin_viewManager
from tracker.models.messages_view import Messages_view, Messages_viewManager


@JsonResponseWithException()
def Messages_view_Fetch(request):
    response = DSResponse(
        request=request,
        data=Messages_view.objects.
            select_related('user', 'state', 'theme', 'to_whom', 'tma_user', 'importance').
            # filter().
            filter().
            get_range_rows1(
            request=request,
            function=Messages_viewManager.getRecord
        ),
        status=RPCResponseConstant.statusSuccess).response
    return JsonResponse(response)


@JsonResponseWithException()
def Messages_view_FetchAdmin(request):
    _response = DSRequest(request)
    print(_response.get_criteria())
    response = DSResponse(
        request=request,
        data=Messages_admin_view.objects.
            select_related('user', 'state', 'theme', 'to_whom', 'importance').
            filter().
            get_range_rows1(
            request=request,
            function=Messages_admin_viewManager.getRecord
        ),
        status=RPCResponseConstant.statusSuccess).response
    return JsonResponse(response)


@JsonResponseWithException()
def Messages_view_Add(request):
    return JsonResponse(DSResponseAdd(data=Messages.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Messages_view_Update(request):
    return JsonResponse(DSResponseUpdate(data=Messages.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Messages_view_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Messages.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Messages_view_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Messages.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Messages_view_Info(request):
    return JsonResponse(DSResponse(request=request, data=Messages.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)



from bitfield import BitHandler
from django.core.serializers.json import DjangoJSONEncoder


class JSONEncoder(DjangoJSONEncoder):
    """
    JSONEncoder subclass that knows how to encode date/time, decimal types, and
    UUIDs.
    """

    def default(self, o):
        if isinstance(o, BitHandler):
            return o._value
        else:
            return super().default(o)



import os
from typing import Text
from uuid import UUID


def getOrElse(default, value):
    if value:
        return value
    else:
        return default


blue = "blue"
closed = "closed"
create = "create"
deleted = "deleted"
doing = "doing"
formirovanie="formirovanie"
green = "green"
made = "made"
new = "new"
new_man = "new_man"
red = "red"
restarted = "restarted"
started = "started"
stoped = "stoped"
transferred = "transferred"
update = "update"
value_odd = "value_odd"

name_new="Новый"
name_new_s="Новый (с)"
name_new_h="Новый (р)"
name_started="Запущен"
name_restarted="Запущен (повторно)"
name_transferred="Назначенный"
name_stoped="Остановлен"
name_doing="Выполнен"
name_closed="Закрыт"
name_formirovanie="Формирование"
name_handmade="Системный (ручное формирование)"


def blinkString(text, blink=True, color="black", bold=False) -> Text:
    if blink:
        res = f'<div class="blink"><strong><font color="{color}"</font>{text}</strong></div>'
    else:
        res = f'<div><strong><font color="{color}"</font>{text}</strong></div>'

    if bold == True:
        return f'<b>{res}</b>'
    else:
        return res


def uuid5():
    return UUID(bytes=os.urandom(16), version=4)

def uuid4():
    return str(UUID(bytes=os.urandom(16), version=4)).upper().replace('-', '_')



# Generated by Django 2.2 on 2019-04-14 09:25

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0021_auto_20190414_0920'),
    ]

    operations = [
        migrations.AlterField(
            model_name='history',
            name='deleted_at',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления'),
        ),
        migrations.AlterField(
            model_name='history',
            name='lastmodified',
            field=models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление'),
        ),
        migrations.AlterField(
            model_name='params',
            name='deleted_at',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления'),
        ),
        migrations.AlterField(
            model_name='params',
            name='lastmodified',
            field=models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление'),
        ),
        migrations.AlterField(
            model_name='user',
            name='deleted_at',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления'),
        ),
        migrations.AlterField(
            model_name='user',
            name='lastmodified',
            field=models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление'),
        ),
        migrations.AlterField(
            model_name='user_permission',
            name='deleted_at',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления'),
        ),
        migrations.AlterField(
            model_name='user_permission',
            name='lastmodified',
            field=models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление'),
        ),
        migrations.AlterField(
            model_name='usergroup',
            name='deleted_at',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления'),
        ),
        migrations.AlterField(
            model_name='usergroup',
            name='lastmodified',
            field=models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление'),
        ),
        migrations.AlterField(
            model_name='usergroup_permission',
            name='deleted_at',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления'),
        ),
        migrations.AlterField(
            model_name='usergroup_permission',
            name='lastmodified',
            field=models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление'),
        ),
        migrations.AlterField(
            model_name='widgets_trees',
            name='deleted_at',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления'),
        ),
        migrations.AlterField(
            model_name='widgets_trees',
            name='lastmodified',
            field=models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление'),
        ),
    ]



import logging

from django.db import transaction
from django.db.models import PositiveIntegerField
from django.forms import model_to_dict

from clndr.models.calendars import Calendars
from clndr.models.shift_day import Shift_day
from clndr.models.shifts import Shifts
from isc_common import delAttr, setAttr
from isc_common.fields.related import ForeignKeyProtect, ForeignKeyCascade
from isc_common.http.DSRequest import DSRequest
from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsManager, CommonManagetWithLookUpFieldsQuerySet
from isc_common.models.audit import AuditModel

logger = logging.getLogger(__name__)


class Calendar_shifts_daysQuerySet(CommonManagetWithLookUpFieldsQuerySet):

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)

    def delete(self):
        with transaction.atomic():
            for item in self:
                try:
                    item.monday.delete()
                except Exception as ex:
                    logger.warning(ex)

                try:
                    item.tuesday.delete()
                except Exception as ex:
                    logger.warning(ex)

                try:
                    item.wednesday.delete()
                except Exception as ex:
                    logger.warning(ex)

                try:
                    item.thursday.delete()
                except Exception as ex:
                    logger.warning(ex)

                try:
                    item.friday.delete()
                except Exception as ex:
                    logger.warning(ex)

                try:
                    item.saturday.delete()
                except Exception as ex:
                    logger.warning(ex)

                try:
                    item.sunday.delete()
                except Exception as ex:
                    logger.warning(ex)


            return super().delete()


class Calendar_shifts_daysManager(CommonManagetWithLookUpFieldsManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'shift_id': record.shift.id,
            'shift__name': record.shift.name,
            'shift__description': record.shift.name,
            'shiftorder': record.shiftorder,

            'monday_id': record.monday.id,
            'monday__info': record.monday.info,

            'tuesday_id': record.tuesday.id,
            'tuesday__info': record.tuesday.info,

            'wednesday_id': record.wednesday.id,
            'wednesday__info': record.wednesday.info,

            'thursday_id': record.thursday.id,
            'thursday__info': record.thursday.info,

            'friday_id': record.friday.id,
            'friday__info': record.friday.info,

            'saturday_id': record.saturday.id,
            'saturday__info': record.saturday.info,

            'sunday_id': record.sunday.id,
            'sunday__info': record.sunday.info,
        }
        return res

    def get_queryset(self):
        return Calendar_shifts_daysQuerySet(self.model, using=self._db)

    def createFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()
        delAttr(_data, 'monday__info')
        delAttr(_data, 'tuesday__info')
        delAttr(_data, 'wednesday__info')
        delAttr(_data, 'thursday__info')
        delAttr(_data, 'friday__info')
        delAttr(_data, 'saturday__info')
        delAttr(_data, 'sunday__info')

        res = super().create(**_data)
        setAttr(data, 'id', res.id)

        return data

    def copyFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        id = data.get('id')

        calendar_shifts_days = Calendar_shifts_days.objects.get(id=id)
        calendar_shifts_days = calendar_shifts_days.copy()

        calendar_shifts_days = model_to_dict(calendar_shifts_days)

        return calendar_shifts_days


class Calendar_shifts_days(AuditModel):
    calendar = ForeignKeyProtect(Calendars)
    shift = ForeignKeyProtect(Shifts)
    shiftorder = PositiveIntegerField()

    monday = ForeignKeyProtect(Shift_day, related_name='monday_rel')
    tuesday = ForeignKeyProtect(Shift_day, related_name='tuesday_rel')
    wednesday = ForeignKeyProtect(Shift_day, related_name='wednesday_rel')
    thursday = ForeignKeyCascade(Shift_day, related_name='thursday_rel')
    friday = ForeignKeyProtect(Shift_day, related_name='friday_rel')
    saturday = ForeignKeyProtect(Shift_day, related_name='saturday_rel')
    sunday = ForeignKeyProtect(Shift_day, related_name='sunday_rel')

    objects = Calendar_shifts_daysManager()

    def copy(self):
        with transaction.atomic():
            kwargs = dict()
            setAttr(kwargs, 'calendar', self.calendar)
            setAttr(kwargs, 'shift', self.shift)
            setAttr(kwargs, 'shiftorder', self.shiftorder)
            setAttr(kwargs, 'monday', self.monday.copy())
            setAttr(kwargs, 'tuesday', self.tuesday.copy())
            setAttr(kwargs, 'wednesday', self.wednesday.copy())
            setAttr(kwargs, 'thursday', self.thursday.copy())
            setAttr(kwargs, 'friday', self.friday.copy())
            setAttr(kwargs, 'saturday', self.saturday.copy())
            setAttr(kwargs, 'sunday', self.sunday.copy())

            return Calendar_shifts_days.objects.create(**kwargs)

    def __str__(self):
        return f"ID={self.id}, " \
               f"\n  shiftorder={self.shiftorder}, " \
               f"\n\n  calendar=[{self.calendar}], " \
               f"\n\n  shift=[{self.shift}], " \
               f"\n\n  monday=[{self.monday}], " \
               f"\n\n  tuesday=[{self.tuesday}], " \
               f"\n\n  wednesday=[{self.wednesday}], " \
               f"\n\n  thursday=[{self.thursday}], " \
               f"\n\n  friday=[{self.friday}], " \
               f"\n\n  friday=[{self.friday}], " \
               f"\n\n  sunday=[{self.sunday}]"

    class Meta:
        verbose_name = 'Рабочие смены по дням'
        ordering = ['shiftorder']



import json
import logging
import sys
import traceback

import websocket
from django.conf import settings

from isc_common import setAttr
from isc_common.http.JSONEncoder import JSONEncoder
from isc_common.ws.progressStack import WebSocketExt

logger = logging.getLogger(__name__)


class WebSocket:
    procedures = []
    logger = logging.getLogger(__name__)

    @staticmethod
    def recv(**kwrags):
        host = kwrags.get('host')
        logger = kwrags.get('logger')

        if not host:
            raise Exception(f'host is not exists.')

        port = kwrags.get('port')
        if not port:
            raise Exception(f'port not exists.')

        if not isinstance(port, int):
            port = int(port)

        channel = kwrags.get('channel')
        if not channel:
            raise Exception(f'channel is not exists.')

        message = kwrags.get('message')

        if not message:
            raise Exception(f'message is not exists.')

        if not isinstance(message, dict):
            raise Exception(f'message is not dict instance')

        key = 'WebSocket.recv'
        settings.LOCKS.acquire(key)
        try:
            url = f'ws://{host}:{port}/ws/{channel}/'
            ws = websocket.create_connection(url)
            ws.send(json.dumps(message))
            settings.LOCKS.release(key)
            return ws, json.loads(ws.recv())
        except Exception as ex:

            exc_info = sys.exc_info()
            message = str(ex)
            stackTrace = traceback.format_exception(*exc_info)

            logging.error(message)

            for x in stackTrace:
                logger.error(x)

            del exc_info
            settings.LOCKS.release(key)
            return None

    @staticmethod
    def send_message(**kwrags):
        host = kwrags.get('host')
        logger = kwrags.get('logger')

        if not host:
            raise Exception(f'host is not exists.')

        port = kwrags.get('port')
        if not port:
            raise Exception(f'port not exists.')

        if not isinstance(port, int):
            port = int(port)

        channel = kwrags.get('channel')
        if not channel:
            raise Exception(f'channel is not exists.')

        message = kwrags.get('message')

        if not isinstance(message, dict):
            if isinstance(message, str):
                message=dict(message=message)
            else:
                raise Exception(f'message is not dict instance')

        try:
            key = 'WebSocket.send_message'
            settings.LOCKS.acquire(key)
            url = f'ws://{host}:{port}/ws/{channel}/'
            ws = websocket.create_connection(url, class_=WebSocketExt)
            ws.send(json.dumps(message, cls=JSONEncoder))
            ws.close()
            settings.LOCKS.release(key)
        except Exception as ex:

            exc_info = sys.exc_info()
            message = str(ex)
            stackTrace = traceback.format_exception(*exc_info)

            logging.error(message)

            for x in stackTrace:
                logger.error(x)

            del exc_info
            settings.LOCKS.release(key)

    @staticmethod
    def full_refresh_grid(grid_id):
        WebSocket.send_message(
            host=settings.WS_HOST,
            port=settings.WS_PORT,
            channel=f'application',
            message=dict(type=grid_id),
            logger=logger
        )

    @staticmethod
    def row_refresh_grid(grid_id, records):
        if not isinstance(records, list):
            if isinstance(records, dict):
                records = [records]
            else:
                raise Exception(f'records must be list or dict')

        WebSocket.send_message(
            host=settings.WS_HOST,
            port=settings.WS_PORT,
            channel=f'application',
            message=dict(type=grid_id, records=records),
            logger=logger
        )

    @staticmethod
    def refresh_progres(grid_id, pk_filed_name, pk_value, progress_field_name, progress_done):

        record = dict()
        setAttr(record, pk_filed_name, pk_value)
        setAttr(record, progress_field_name, progress_done)

        WebSocket.row_refresh_grid(
            grid_id=grid_id,
            records=[record]
        )

    @staticmethod
    def send_warning_message(**kwrags):
        setAttr(kwrags, 'type', 'warn')
        WebSocket.send_typing_message(**kwrags)

    @staticmethod
    def send_info_message(**kwrags):
        setAttr(kwrags, 'type', 'info')
        WebSocket.send_typing_message(**kwrags)

    @staticmethod
    def send_typing_message(**kwrags):

        type = kwrags.get('type')
        if not type:
            raise Exception(f'channel is not type.')

        message = kwrags.get('message')
        if isinstance(message, str):
            message = dict(
                message=message,
                type=type,
            )

        setAttr(kwrags, 'message', message)

        WebSocket.send_message(**kwrags)


def on_message(self, message):
    for proc in self.procedures:
        if hasattr(proc, '__call__'):
            proc(message)


def on_error(self, error):
    self.logger.error(error)


def on_close(self):
    self.logger.debug(f'### closed ###')


def on_open(self):
    self.logger.debug(f'### opened ###')


def remove_proc(self, proc):
    self.procedures.remove(proc)


def append_proc(self, proc):
    self.procedures.append(proc)


def send(self, data):
    if isinstance(data, dict):
        self.webSocket.send(json.dumps(data))
    else:
        raise Exception(f'data is not dict.')


def send_logging(self, msg, type='logging'):
    data = dict(msg=msg, type=type)
    # self.webSocket.send(json.dumps(data))


def __init__(self, **kwargs):
    for k, v in kwargs.items():
        setattr(self, k, v() if callable(v) else v)

    websocket.enableTrace(True)
    self.webSocket = websocket.WebSocketApp(f'ws://{self.host}:{self.port}/ws/{self.channel}/',
                                            on_message=self.on_message,
                                            on_error=self.on_error,
                                            on_close=self.on_close,
                                            on_open=self.on_open
                                            )
    self.webSocket.run_forever()



from django.db import IntegrityError
from django.test import TestCase
from isc_common.auth.models.usergroup import UserGroup


class Test_UserGroup(TestCase):

    def setUp(self):
        self.code = "code"
        self.name = "name"
        self.description = "description"

    def test_1(self):
        UserGroup_item = UserGroup.objects.create(code=self.code, name=self.name, description=self.description)
        UserGroup_item = UserGroup.objects.get(pk=UserGroup_item.pk)
        self.assertEquals(UserGroup_item.code, self.code)
        self.assertEquals(UserGroup_item.name, self.name)
        self.assertEquals(UserGroup_item.description, self.description)

        self.code += '1'
        UserGroup_item = UserGroup.objects.create(code=self.code, name=self.name)
        UserGroup_item = UserGroup.objects.get(pk=UserGroup_item.pk)
        self.assertEquals(UserGroup_item.code, self.code)
        self.assertEquals(UserGroup_item.name, self.name)
        self.assertEquals(UserGroup_item.description, None)

        self.code += '1'
        UserGroup_item = UserGroup.objects.create(code=self.code)
        UserGroup_item = UserGroup.objects.get(pk=UserGroup_item.pk)
        self.assertEquals(UserGroup_item.code, self.code)
        self.assertEquals(UserGroup_item.name, None)
        self.assertEquals(UserGroup_item.description, None)

    def test_2(self):
        with self.assertRaises(AssertionError):
            UserGroup_item = UserGroup.objects.create()

    def test_3(self):
        with self.assertRaises(IntegrityError):
            UserGroup_item = UserGroup.objects.create(code=self.code)
            UserGroup_item = UserGroup.objects.create(code=self.code)




# Generated by Django 2.2.4 on 2019-08-21 16:40

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0016_remove_holidays_props'),
    ]

    operations = [
        migrations.AddField(
            model_name='holidays',
            name='day_type',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='clndr.Day_types'),
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-26 10:42

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('one_c', '0003_files_1c'),
    ]

    operations = [
        migrations.AlterField(
            model_name='files_1c',
            name='props',
            field=bitfield.models.BitField((('imp', 'Импорт'), ('exp', 'Экспорт')), db_index=True, default=None),
        ),
    ]



from django.urls import path

from clndr.views import shifts

urlpatterns = [

    path('Shifts/Fetch/', shifts.Shifts_Fetch),
    path('Shifts/Add', shifts.Shifts_Add),
    path('Shifts/Update', shifts.Shifts_Update),
    path('Shifts/Remove', shifts.Shifts_Remove),
    path('Shifts/Lookup/', shifts.Shifts_Lookup),
    path('Shifts/Info/', shifts.Shifts_Info),
    path('Shifts/Copy', shifts.Shifts_Copy),

]



# Generated by Django 2.2.4 on 2019-08-05 22:39

from django.conf import settings
from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0032_auto_20190805_2238'),
    ]

    operations = [
        migrations.AlterField(
            model_name='messages',
            name='user',
            field=isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
    ]



from django.utils.translation import ugettext_lazy as _

import logging

from isc_common.auth.models.user import User
from isc_common.fields.related import ForeignKeyCascade
from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet

logger = logging.getLogger(__name__)


class Message_user_2_userQuerySet(AuditQuerySet):
    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Message_user_2_userManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'user_reciver_id': record.user_reciver.id,
            'user_reciver__username': record.user_reciver.username,
            'user_reciver__first_name': record.user_reciver.first_name,
            'user_reciver__last_name': record.user_reciver.last_name,
            'user_reciver__middle_name': record.user_reciver.middle_name,
        }
        return res

    def get_queryset(self):
        return Message_user_2_userQuerySet(self.model, using=self._db)


class Message_user_2_user(AuditModel):
    user_sender = ForeignKeyCascade(User, related_name='user_sender')
    user_reciver = ForeignKeyCascade(User, related_name='user_reciver')

    objects = Message_user_2_userManager()

    def __str__(self):
        return f"{self.id}"

    class Meta:
        verbose_name = 'Доступ к получателям сообщений'



import urllib
from django import test

from isc_common.http.RPCResponse import RPCResponseConstant


class TestCase(test.TestCase):
    def assertSuccessResponse(self, response):
        if response.container_data['response']['status'] == RPCResponseConstant.statusFailure:
            message = response.container_data['response']['data']['error']['message']

            stackTrace = response.container_data['response']['data']['error']['stackTrace']
            stackTrace = stackTrace.replace(', \'  ','')
            stackTrace = stackTrace.encode().decode('unicode-escape')

            raise self.failureException(f"\nmessage: {message}\nstackTrace:{stackTrace}")

    def assertFailureResponse(self, response, message=None, stackTrace=None):
        if response.container_data['response']['status'] == RPCResponseConstant.statusFailure:
            _message = response.container_data['response']['data']['error']['message']

            if message and message != _message:
                raise self.failureException(f"message: {message} != {_message}")

            _stackTrace = response.container_data['response']['data']['error']['stackTrace']
            if stackTrace and stackTrace != _stackTrace:
                raise self.failureException(f"stackTrace: {stackTrace} != {_stackTrace}")

        else:
            raise self.failureException(f"response not failure status.")



# Generated by Django 2.2.4 on 2019-08-29 10:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0068_day_types_length'),
    ]

    operations = [
        migrations.AlterField(
            model_name='shift_day_relax',
            name='relaxorder',
            field=models.PositiveIntegerField(default=None),
        ),
    ]



from django.urls import path

from tracker.views import messages_theme_users_access

urlpatterns = [

    path('MessagesThemeUsersAccess/Fetch/', messages_theme_users_access.MessagesThemeUsersAccess_Fetch),
    path('MessagesThemeUsersAccess/Add', messages_theme_users_access.MessagesThemeUsersAccess_Add),
    path('MessagesThemeUsersAccess/Update', messages_theme_users_access.MessagesThemeUsersAccess_Update),
    path('MessagesThemeUsersAccess/Remove', messages_theme_users_access.MessagesThemeUsersAccess_Remove),
    path('MessagesThemeUsersAccess/Lookup/', messages_theme_users_access.MessagesThemeUsersAccess_Lookup),
    path('MessagesThemeUsersAccess/Info/', messages_theme_users_access.MessagesThemeUsersAccess_Info),
    path('MessagesThemeUsersAccess/Copy', messages_theme_users_access.MessagesThemeUsersAccess_Copy),

]



from isc_common.auth.managers.user_manager import UserManager
from isc_common.auth.models.user import User
from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse


@JsonResponseWithException(printing=False)
def User_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=User.objects.
                exclude(id=0).
                distinct().
                get_range_rows11(
                request=request,
                function=UserManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException(printing=False)
def User_FetchExclBots(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=User.objects.
                exclude(props=User.props.bot).
                exclude(id=0).
                distinct().
                get_range_rows11(
                request=request,
                function=UserManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def User_Add(request):
    return JsonResponse(DSResponseAdd(data=User.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException(printing=False)
def User_Update(request):
    return JsonResponse(DSResponseUpdate(data=User.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def User_Remove(request):
    return JsonResponse(DSResponse(request=request, data=User.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def User_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=User.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def User_Info(request):
    return JsonResponse(DSResponse(request=request, data=User.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)



# Generated by Django 3.0.8 on 2020-07-13 05:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0054_auto_20200713_0552'),
    ]

    operations = [
        migrations.AlterField(
            model_name='params',
            name='value',
            field=models.TextField(),
        ),
    ]



import logging

from django.db.models import DateTimeField, BooleanField, UUIDField
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _

from isc_common.auth.models.user import User
from isc_common.bit import IsBitOff
from isc_common.fields.description_field import DescriptionField
from isc_common.fields.related import ForeignKeyProtect, ForeignKeyCascade
from isc_common.http.DSRequest import DSRequest
from isc_common.models.audit import AuditModel
from tracker.models.messages import MessagesManager, MessagesQuerySet
from tracker.models.messages_importance import Messages_importance
from tracker.models.messages_state import Messages_state
from tracker.models.messages_theme import Messages_theme

logger = logging.getLogger(__name__)


class Messages_viewQuerySet(MessagesQuerySet):
    def get_range_rows1(self, request, function=None, distinct_field_names=None):
        request = DSRequest(request=request)
        self.alive_only = request.alive_only
        self.enabledAll = request.enabledAll
        return self.get_range_rows3(start=request.startRow, end=request.endRow, function=function, distinct_field_names=distinct_field_names, json=request.json)


class Messages_viewManager(MessagesManager):

    @staticmethod
    def getRecord(record, enabled=None):
        res = {
            'date_create': record.date_create,
            'deliting': record.deliting,
            'editing': record.editing,
            'guid': str(record.guid).upper() if record.guid else None,
            'id': record.id,
            'isFolder': record.isFolder,
            'lastmodified': record.lastmodified,
            'message': record.message,
            'parent_id': record.parent_id,
            'state__name': record.state.name,
            'state_id': record.state.id,
            'theme__full_name': record.theme.full_name,
            'theme__name': record.theme.full_name,
            'theme_id': record.theme.id,
            'importance_id': record.importance.id if record.importance else None,
            'importance__code': record.importance.code if record.importance else None,
            'importance__name': record.importance.name if record.importance else None,
            'to_whom_id': record.to_whom.id,
            'to_whom__short_name': record.to_whom.get_short_name,
            'user__short_name': record.user.get_short_name,
            'user__username': record.user.username,
            'user_id': record.user.id if record.user else None,
            'enabled': IsBitOff(record.state.props, 0),
        }
        return res

    def get_queryset(self):
        return Messages_viewQuerySet(self.model, using=self._db)


class Messages_view(AuditModel):
    guid = UUIDField(blank=True, null=True)
    date_create = DateTimeField(verbose_name='Дата записи', db_index=True, default=timezone.now)
    isFolder = BooleanField(default=False)
    all_content = BooleanField(default=False)
    message = DescriptionField(_('Тело задач'), null=False, blank=False)
    parent = ForeignKeyProtect('self', blank=True, null=True)
    state = ForeignKeyProtect(Messages_state)
    theme = ForeignKeyProtect(Messages_theme)
    user = ForeignKeyCascade(User, related_name='user_view')
    tma_user = ForeignKeyCascade(User, related_name='tma_user_view')
    to_whom = ForeignKeyCascade(User, related_name='user_to_whom')
    importance = ForeignKeyProtect(Messages_importance, blank=True, null=True)

    def __str__(self):
        return f'id: {self.id}, guid: {str(self.guid).upper()}, message: {self.message}, date_create: {self.date_create}, user: [{self.user}], theme: [{self.theme}], state: [{self.state}]'

    objects = Messages_viewManager()

    class Meta:
        managed = False
        db_table = 'tracker_messages_view'
        verbose_name = 'Сообщения'



# Generated by Django 2.2.7 on 2019-11-16 13:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('history', '0009_auto_20191116_1312'),
    ]

    operations = [
        migrations.AlterField(
            model_name='history',
            name='ip_address',
            field=models.CharField(max_length=20),
        ),
        migrations.AlterField(
            model_name='history',
            name='user_agent',
            field=models.CharField(max_length=255),
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-24 12:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0046_auto_20190824_1257'),
    ]

    operations = [
        migrations.AlterField(
            model_name='shift_day',
            name='numdayinweek',
            field=models.PositiveIntegerField(unique=True),
        ),
    ]



import logging
import os
from datetime import datetime
from urllib.parse import urljoin

from django.conf import settings
from django.core.files import File, locks
from django.core.files.move import file_move_safe
from django.core.files.storage import Storage
from django.core.signals import setting_changed
from django.utils import timezone
from django.utils._os import safe_join
from django.utils.deconstruct import deconstructible
from django.utils.encoding import filepath_to_uri
from django.utils.functional import cached_property

logger = logging.getLogger(__name__)


@deconstructible
class CryptoFileSystemStorage(Storage):
    """
    Standard filesystem storage
    """

    def __init__(self, location=None, base_url=None, file_permissions_mode=None, directory_permissions_mode=None):
        self._location = location
        self._base_url = base_url
        self._file_permissions_mode = file_permissions_mode
        self._directory_permissions_mode = directory_permissions_mode
        setting_changed.connect(self._clear_cached_properties)

    def _clear_cached_properties(self, setting, **kwargs):
        """Reset setting based property values."""
        if setting == 'FILES_STORE':
            self.__dict__.pop('base_location', None)
            self.__dict__.pop('location', None)
        elif setting == 'MEDIA_URL':
            self.__dict__.pop('base_url', None)
        elif setting == 'FILE_UPLOAD_PERMISSIONS':
            self.__dict__.pop('file_permissions_mode', None)
        elif setting == 'FILE_UPLOAD_DIRECTORY_PERMISSIONS':
            self.__dict__.pop('directory_permissions_mode', None)

    def _value_or_setting(self, value, setting):
        return setting if value == None else value

    @cached_property
    def base_location(self):
        return self._value_or_setting(self._location, settings.FILES_STORE)

    @cached_property
    def location(self):
        return os.path.abspath(self.base_location)

    @cached_property
    def base_url(self):
        if self._base_url != None and not self._base_url.endswith('/'):
            self._base_url += '/'
        return self._value_or_setting(self._base_url, settings.MEDIA_URL)

    @cached_property
    def file_permissions_mode(self):
        return self._value_or_setting(self._file_permissions_mode, settings.FILE_UPLOAD_PERMISSIONS)

    @cached_property
    def directory_permissions_mode(self):
        return self._value_or_setting(self._directory_permissions_mode, settings.FILE_UPLOAD_DIRECTORY_PERMISSIONS)

    def _open(self, name, mode='rb'):
        return File(open(self.path(name), mode))

    def _save(self, name, content):
        full_path = self.path(name)

        directory = os.path.dirname(full_path)
        if not os.path.exists(directory):
            try:
                if self.directory_permissions_mode != None:
                    # os.makedirs applies the global umask, so we reset it,
                    # for consistency with file_permissions_mode behavior.
                    old_umask = os.umask(0)
                    try:
                        os.makedirs(directory, self.directory_permissions_mode)
                    finally:
                        os.umask(old_umask)
                else:
                    os.makedirs(directory)
            except FileExistsError:
                pass
        if not os.path.isdir(directory):
            raise IOError("%s exists and is not a directory." % directory)

        while True:
            try:
                # This file has a file path that we can move.
                if hasattr(content, 'temporary_file_path'):
                    file_move_safe(content.temporary_file_path(), full_path)

                # This is a normal uploadedfile that we can stream.
                else:
                    # This fun binary flag incantation makes os.open throw an
                    # OSError if the file already exists before we open it.
                    flags = (os.O_WRONLY | os.O_CREAT | os.O_EXCL |
                             getattr(os, 'O_BINARY', 0))
                    # The current umask value is masked out by os.open!
                    fd = os.open(full_path, flags, 0o666)
                    _file = None
                    try:
                        locks.lock(fd, locks.LOCK_EX)
                        for chunk in content.chunks():
                            if _file == None:
                                mode = 'wb' if isinstance(chunk, bytes) else 'wt'
                                _file = os.fdopen(fd, mode)
                            _file.write(chunk)
                    finally:
                        locks.unlock(fd)
                        if _file != None:
                            _file.close()
                        else:
                            os.close(fd)
            except FileExistsError:
                # A new name is needed if the file exists.
                name = self.get_available_name(name)
                full_path = self.path(name)
            else:
                # OK, the file save worked. Break out of the loop.
                break

        if self.file_permissions_mode != None:
            os.chmod(full_path, self.file_permissions_mode)

        # Store filenames with forward slashes, even on Windows.
        return name.replace('\\', '/')

    def delete(self, name):
        assert name, "The name argument is not allowed to be empty."
        name = self.path(name)
        # If the file or directory exists, delete it from the filesystem.
        try:
            if os.path.isdir(name):
                os.rmdir(name)
            else:
                os.remove(name)
        except FileNotFoundError as e:
            raise e
            # logger.warning(f'{name}: not found.')


    def exists(self, name):
        return os.path.exists(self.path(name))

    def listdir(self, path):
        path = self.path(path)
        directories, files = [], []
        for entry in os.listdir(path):
            if os.path.isdir(os.path.join(path, entry)):
                directories.append(entry)
            else:
                files.append(entry)
        return directories, files

    def path(self, name):
        return safe_join(self.location, name)

    def size(self, name):
        return os.path.getsize(self.path(name))

    def url(self, name):
        if self.base_url == None:
            raise ValueError("This file is not accessible via a URL.")
        url = filepath_to_uri(name)
        if url != None:
            url = url.lstrip('/')
        return urljoin(self.base_url, url)

    def _datetime_from_timestamp(self, ts):
        """
        If timezone support is enabled, make an aware datetime object in UTC;
        otherwise make a naive one in the local timezone.
        """
        if settings.USE_TZ:
            # Safe to use .replace() because UTC doesn't have DST
            return datetime.utcfromtimestamp(ts).replace(tzinfo=timezone.utc)
        else:
            return datetime.fromtimestamp(ts)

    def get_accessed_time(self, name):
        return self._datetime_from_timestamp(os.path.getatime(self.path(name)))

    def get_created_time(self, name):
        return self._datetime_from_timestamp(os.path.getctime(self.path(name)))

    def get_modified_time(self, name):
        return self._datetime_from_timestamp(os.path.getmtime(self.path(name)))



from django.urls import path
from isc_common.views import params

urlpatterns = [

    path('Params/Fetch/', params.Params_Fetch),
    path('Params/Add', params.Params_Add),
    path('Params/Update', params.Params_Update),
    path('Params/Update1', params.Params_Update1),
    path('Params/Update2', params.Params_Update1),
    path('Params/Update3', params.Params_Update1),
    path('Params/Update4', params.Params_Update1),
    path('Params/Update5', params.Params_Update1),
    path('Params/Update6', params.Params_Update1),
    path('Params/Update7', params.Params_Update1),
    path('Params/Update8', params.Params_Update1),
    path('Params/Update9', params.Params_Update1),
    path('Params/Update10', params.Params_Update1),
    path('Params/Remove', params.Params_Remove),
    path('Params/Remove1', params.Params_Remove1),
    path('Params/Lookup', params.Params_Lookup),
    path('Params/Info/', params.Params_Info),
    path('Params/Get/', params.Params_Get),
    path('Params/GetKeyValue/', params.Params_GetKeyValue),

]



from isc_common.auth.models.user import User
from isc_common.auth.models.usergroup import UserGroup
from isc_common.http.DSRequest import DSRequest
from isc_common.http.RPCResponse import RPCResponseConstant


class Link2Group(DSRequest):
    def __init__(self, request):
        DSRequest.__init__(self, request)
        data = self.get_data()
        user_ids = data.get('user_ids', None)
        group_ids = data.get('group_ids', None)

        if user_ids and group_ids:
            if not isinstance(user_ids, list):
                user_ids = [user_ids]

            if not isinstance(group_ids, list):
                group_ids = [group_ids]

            for user_id in user_ids:
                user = User.objects.get(id=user_id)
                for group_id in group_ids:
                    try:
                        group = UserGroup.objects.get(id=group_id)
                        user.usergroup.add(group)
                    except:
                        pass

        self.response = dict(status=RPCResponseConstant.statusSuccess, user_ids=user_ids)



import io
import json
import subprocess
import sys


def makeInfo():
    data = json.loads(io.open('./package.json', encoding='utf-8').read())

    info = []
    info.append(dict(libName='Разработка :', libVersion='"АО ИВЦ \"Информ\" (info@ivc-inform.ru)'))
    info.append(dict(libName='Версия :', libVersion=data.get('version')))
    info.append(dict(libName='Авторы :', libVersion=data.get('author')))
    info.append(dict(libName='Контакты :', libVersion="+7 (495) 997-91-75"))

    installed_packages = subprocess.check_output([sys.executable, '-m', 'pip', 'freeze'])
    installed_packages_list = sorted([(item[: item.find('==')], item[item.find('==') + 2:]) for item in
                                      installed_packages.decode("utf-8").split('\n')])

    for (libName, libVersion) in installed_packages_list:
        info.append(dict(libName=libName, libVersion=libVersion))

    file = open('MakeAboutData.js', 'w')
    str = f"simpleSyS.aboutData = {json.dumps(info)}"
    file.write(str)
    file.close()



from django.urls import path

from twits.views import chat_user_user

urlpatterns = [

    path('Chat_user_user/Fetch/', chat_user_user.Chat_user_user_Fetch),
    path('Chat_user_user/Add', chat_user_user.Chat_user_user_Add),
    path('Chat_user_user/Update', chat_user_user.Chat_user_user_Update),
    path('Chat_user_user/Remove', chat_user_user.Chat_user_user_Remove),
    path('Chat_user_user/Lookup/', chat_user_user.Chat_user_user_Lookup),
    path('Chat_user_user/Info/', chat_user_user.Chat_user_user_Info),
    path('Chat_user_user/Copy', chat_user_user.Chat_user_user_Copy),

]



# Generated by Django 2.2.4 on 2019-08-28 20:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0067_auto_20190826_0659'),
    ]

    operations = [
        migrations.AddField(
            model_name='day_types',
            name='length',
            field=models.TimeField(blank=True, null=True),
        ),
    ]



import json
import logging
from json import JSONDecodeError

from isc_common.string import StrToBytes, BytesToStr

logger = logging.getLogger(__name__)


def BytesToJson(mybytes):
    try:
        mystring = BytesToStr(mybytes)
        _json = json.loads(mystring)
        return _json
    except JSONDecodeError:
        return dict()


def JsonToBytes(myjson):
    mystring = json.dumps(myjson)
    bytes = StrToBytes(mystring)
    return bytes


def JsonToStr(myjson, indent=4, sort_keys=True):
    if isinstance(myjson, dict):
        if indent == 0:
            return json.dumps(myjson, indent=indent, sort_keys=sort_keys).replace('\n','')
        else:
            return json.dumps(myjson, indent=indent, sort_keys=sort_keys)
    else:
        raise Exception(f'{myjson} must by dict')


def StrToJson(mystring):
    if isinstance(mystring, str):
        try:
            return json.loads(mystring)
        except JSONDecodeError as ex:
            logger.error(ex)
            return dict()
    elif isinstance(mystring, dict):
        return mystring
    else:
        raise Exception(f'{mystring} must be str')



import logging

from bitfield import BitField
from django.db import transaction

from isc_common import setAttr, delAttr
from isc_common.auth.models.user import User
from isc_common.bit import IsBitOn
from isc_common.fields.related import ForeignKeyCascade
from isc_common.http.DSRequest import DSRequest
from isc_common.models.audit import AuditModel, AuditQuerySet, AuditManager
from tracker.models.messages_theme import Messages_theme

logger = logging.getLogger(__name__)


class MessagesThemeUsersAccessQuerySet(AuditQuerySet):
    pass


class MessagesThemeUsersAccessManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'theme_id': record.theme.id,
            'user_id': record.user.id,
            'user__username': record.user.username,
            'user__first_name': record.user.first_name,
            'user__last_name': record.user.last_name,
            'user__email': record.user.email,
            'user__middle_name': record.user.middle_name,
            'props': record.user.props._value,
            'all_content': IsBitOn(record.props._value, 0)
        }
        return res

    def updateFromRequest(self, request, removed=None, function=None):
        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()
        props = _data.get('props')

        if data.get('all_content') == True:
            props |= MessagesThemeUsersAccess.props.all_content
        else:
            props &= ~MessagesThemeUsersAccess.props.all_content

        setAttr(_data, 'props', props)
        delAttr(_data, 'id')
        delAttr(_data, 'user__username')
        delAttr(_data, 'user__first_name')
        delAttr(_data, 'user__last_name')
        delAttr(_data, 'user__email')
        delAttr(_data, 'user__middle_name')
        delAttr(_data, 'all_content')

        res = super().filter(id=data.get('id')).update(**_data)
        return data

    def createFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        user_ids = data.get('user_ids', None)
        context_ids = data.get('context_ids', None)

        with transaction.atomic():
            if user_ids and context_ids:
                if not isinstance(user_ids, list):
                    user_ids = [user_ids]

                if not isinstance(context_ids, list):
                    context_ids = [context_ids]

                for user_id in user_ids:
                    for context_id in context_ids:
                        res, _ = super().get_or_create(user_id=user_id, theme_id=context_id)

            return data

    def get_queryset(self):
        return MessagesThemeUsersAccessQuerySet(self.model, using=self._db)


class MessagesThemeUsersAccess(AuditModel):
    theme = ForeignKeyCascade(Messages_theme)
    user = ForeignKeyCascade(User)
    props = BitField(flags=(
        ('all_content', 'Доступ ко всем делам данной темы'),  # 1
    ), default=0, db_index=True)

    objects = MessagesThemeUsersAccessManager()

    def __str__(self):
        return f"{self.id}"

    class Meta:
        verbose_name = 'Доступы к темам'



import calendar
import datetime
import uuid

from django.conf import settings

from clndr.calendar_event import CalendarEvent, CalendarEventLinkedList
from clndr.models.calendar_shifts_days import Calendar_shifts_days
from clndr.models.calendars import Calendars
from clndr.models.holidays import Holidays
from clndr.models.shift_day_relax import Shift_day_relax
from isc_common.auth.models.user import User
from isc_common.common import uuid4
from isc_common.ws.progressStack import ProgressStack


class Timeline:
    logger = None
    calendar = None
    calendar_id = None
    head = None
    top_ce = None
    cur_ce = None

    def __init__(self, *args, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v() if callable(v) else v)

        if not self.calendar and not isinstance(self.calendar, Calendars):
            if self.calendar_id:
                self.calendar = Calendars.objects.get(id=self.calendar_id)
        else:
            self.calendar_id = self.calendar.id

        if not self.calendar:
            raise Exception(f'Не выбран календарь.')

        if not self.logger:
            raise Exception(f'Не установлен логер.')

        self.shiftsList = CalendarEventLinkedList()
        self.relaxesList = CalendarEventLinkedList()

        self.shifts = Calendar_shifts_days.objects.filter(calendar=self.calendar)

        self.red_label_days = CalendarEventLinkedList(query=Holidays.objects.filter(calendar=self.calendar, daytype__isholiday=True).order_by(*['begindate']))
        # self.red_label_days.print('red_label_days')

        self.shorted_days = CalendarEventLinkedList(query=Holidays.objects.exclude(calendar=self.calendar, daytype__length__isnull=True))
        # self.shorted_days.print('shorted_days')

    def select_build_shift_day(self, shift, dayweek):
        if dayweek == 0:
            return shift.monday
        elif dayweek == 1:
            return shift.tuesday
        elif dayweek == 2:
            return shift.wednesday
        elif dayweek == 3:
            return shift.thursday
        elif dayweek == 4:
            return shift.friday
        elif dayweek == 5:
            return shift.saturday
        elif dayweek == 6:
            return shift.sunday

    def dayweek_name(self, dayweek):
        dayweek = dayweek.weekday()
        if dayweek == 0:
            return 'Понедельник'
        elif dayweek == 1:
            return 'Вторник'
        elif dayweek == 2:
            return 'Среда'
        elif dayweek == 3:
            return 'Четверг'
        elif dayweek == 4:
            return 'Пятница'
        elif dayweek == 5:
            return 'Суббота'
        elif dayweek == 6:
            return 'Воскресенье'

    def check_next_day(self, begindatetime, endtime):
        if not isinstance(begindatetime, datetime.datetime):
            raise Exception('begindatetime is not datetime.datetime type')

        if not isinstance(endtime, datetime.time):
            raise Exception('endtime is not datetime.time type')

        if begindatetime.hour > endtime.hour or \
                begindatetime.hour == endtime.hour and begindatetime.minute > endtime.minute or \
                begindatetime.hour == endtime.hour and begindatetime.minute == endtime.minute and begindatetime.second > endtime.second or \
                begindatetime.hour == endtime.hour and begindatetime.minute == endtime.minute and begindatetime.second == endtime.second and begindatetime.microsecond > endtime.microsecond:  # or \
            # begindatetime.hour == endtime.hour and begindatetime.minute == endtime.minute and begindatetime.second == endtime.second and begindatetime.microsecond == endtime.microsecond:

            _, month_range = calendar.monthrange(begindatetime.year, begindatetime.month)
            if month_range < begindatetime.day + 1:
                if begindatetime.month + 1 > 12:
                    return datetime.datetime(year=begindatetime.year + 1, month=1, day=1, hour=endtime.hour, minute=endtime.minute, second=endtime.second)
                else:
                    return datetime.datetime(year=begindatetime.year, month=begindatetime.month + 1, day=1, hour=endtime.hour, minute=endtime.minute, second=endtime.second)
            else:
                return datetime.datetime(year=begindatetime.year, month=begindatetime.month, day=begindatetime.day + 1, hour=endtime.hour, minute=endtime.minute, second=endtime.second)
        else:
            return datetime.datetime(year=begindatetime.year, month=begindatetime.month, day=begindatetime.day, hour=endtime.hour, minute=endtime.minute, second=endtime.second)

    def build_shift_day(self, shift, shiftday):
        startDate = self.curdate
        _, month_range = calendar.monthrange(startDate.year, startDate.month)

        red_label_day = self.red_label_days.find(startDate, lambda item, x: x.begindate <= item and x.enddate > item)
        if red_label_day:
            if month_range < startDate.day + 1:
                if red_label_day.month + 1 > 12:
                    self.shiftsList.add(CalendarEvent(
                        id=uuid4(),
                        backgroundColor=red_label_day.daytype.color,
                        endDate=datetime.datetime(year=startDate.year + 1, month=1, day=1),
                        isredlabelday=True,
                        lane=self.calendar.code,
                        name=f'{red_label_day.name}',
                        startDate=startDate,
                    ))
                else:
                    self.shiftsList.add(CalendarEvent(
                        id=uuid4(),
                        backgroundColor=red_label_day.daytype.color,
                        endDate=datetime.datetime(year=startDate.year, month=startDate.month + 1, day=1),
                        isredlabelday=True,
                        lane=self.calendar.code,
                        name=f'{red_label_day.name}',
                        startDate=startDate,
                    ))
            else:
                self.shiftsList.add(CalendarEvent(
                    id=uuid4(),
                    backgroundColor=red_label_day.daytype.color,
                    endDate=datetime.datetime(year=startDate.year, month=startDate.month, day=startDate.day + 1),
                    isredlabelday=True,
                    lane=self.calendar.code,
                    name=f'{red_label_day.name}',
                    startDate=startDate,
                ))

            # self.shiftsList.print('shiftsList')

            return False

        if shiftday.daytype.isholiday:
            if month_range < startDate.day + 1:
                if startDate.month + 1 > 12:
                    endDate = datetime.datetime(year=startDate.year + 1, month=1, day=1)
                else:
                    endDate = datetime.datetime(year=startDate.year, month=startDate.month + 1, day=1)
            else:
                endDate = datetime.datetime(year=startDate.year, month=startDate.month, day=startDate.day + 1)
        else:
            shorted_day = self.shorted_days.find(startDate, lambda item, x: x.begindate <= item and x.enddate > item)

            startDate = self.check_next_day(self.curdate, shiftday.begintime)
            if shorted_day:
                endDate = self.check_next_day(startDate, shiftday.endtime)

                length = (endDate - startDate) - datetime.timedelta(hours=shorted_day.daytype.length.hour + 1, minutes=shorted_day.daytype.length.minute, seconds=shorted_day.daytype.length.second)
                endDate = endDate - length
            else:
                endDate = self.check_next_day(startDate, shiftday.endtime)

        # print(f'curdate: {self.curdate}')
        # print(f'startDate: {startDate}')
        # print(f'endDate: {endDate}')

        self.shiftsList.add(CalendarEvent(
            id=uuid4(),
            backgroundColor=shiftday.daytype.color,
            description=f'{shift.shift.name} {shiftday.description if shiftday.description else ""}',
            endDate=endDate,
            isholiday=shiftday.daytype.isholiday,
            isworkday=not shiftday.daytype.isholiday,
            lane=self.calendar.code,
            name=shiftday.name,
            startDate=startDate,
        ))

        # self.shiftsList.print('shiftsList')

        if not shiftday.daytype.isholiday:
            for shift_day_relax in Shift_day_relax.objects.filter(shiftday=shiftday).order_by(*['relaxorder']):
                startDateRelax = self.check_next_day(startDate, shift_day_relax.begintime)
                endDateRelax = self.check_next_day(startDateRelax, shift_day_relax.endtime)

                # print(f'startDateRelax: {startDateRelax}')
                # print(f'endDateRelax: {endDateRelax}')

                if endDateRelax > endDate:
                    raise Exception(f'Неправильнно заполнен {shift_day_relax} конечное время {endDateRelax} не может быть больше конечного времени смены {endDate}.')

                if startDate < startDate:
                    raise Exception(f'Неправильнно заполнен {shift_day_relax} начальное время {startDate} не может быть меньше начального времени смены {startDate}.')

                # Данные перекура
                self.relaxesList.add(CalendarEvent(
                    id=uuid4(),
                    backgroundColor=shift_day_relax.color,
                    description=f'{shift.shift.name} ({shift_day_relax.name})',
                    endDate=endDateRelax,
                    isholiday=shift_day_relax.shiftday.daytype.isholiday,
                    lane=self.calendar.code,
                    name=shift_day_relax.shiftday.name,
                    startDate=startDateRelax,
                ))

            # self.shiftsList.print('shiftsList')
            return True
        else:
            return False

    def build(self, data):
        self.curdate = self.calendar.begindate
        enddate = self.calendar.enddate
        if data.get('user_id') == None or not isinstance(data.get('user_id'), int):
            raise Exception(f'user_id not found')
        user = User.objects.get(id=data.get('user_id'))

        progress = ProgressStack(
            host=settings.WS_HOST,
            port=settings.WS_PORT,
            channel=f'common_{user.username}',
            user_id=user.id
        )

        id_progress = f'launch_{user.id}'

        demand_str = f'<h3>Создание календаря : {self.calendar.full_name}</h3>'

        cntAll = (self.calendar.enddate - self.calendar.begindate).days
        progress.show(
            title=f'<b>Выполнено</b>',
            label_contents=demand_str,
            cntAll=cntAll,
            id=id_progress
        )

        cnt = 0

        while self.curdate < enddate:
            dayweek = self.curdate.weekday()
            for dw in range(dayweek, 7):
                for shift in self.shifts:
                    if not self.build_shift_day(shift, self.select_build_shift_day(shift, dw)):
                        break

                _, month_range = calendar.monthrange(self.curdate.year, self.curdate.month)

                if month_range < self.curdate.day + 1:
                    if self.curdate.month + 1 > 12:
                        self.curdate = datetime.datetime(year=self.curdate.year + 1, month=1, day=1)
                    else:
                        self.curdate = datetime.datetime(year=self.curdate.year, month=self.curdate.month + 1, day=1)
                else:
                    self.curdate = datetime.datetime(year=self.curdate.year, month=self.curdate.month, day=self.curdate.day + 1)

                progress.setCntDone(cnt, id_progress)
                cnt += 1

                if self.curdate >= enddate:
                    break

        # self.shiftsList.print('shiftsList')
        self.shiftsList.fusion(self.relaxesList)
        progress.close(id=id_progress)
        # self.shiftsList.print('shifts')
        return self.shiftsList



# Generated by Django 2.2.4 on 2019-08-05 21:59

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0029_message_user_2_user'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Message_user_2_user',
        ),
    ]



from decimal import Decimal
from functools import reduce
from itertools import chain

from bitfield import BitHandler, Bit
from django.db.models import ForeignKey

from isc_common import setAttr


def del_last_zero(value):
    if isinstance(value, str):
        return value

    res = str(value)
    res.split('.')
    cel = res.split('.')[0]
    float = StrToNumber(res.split('.')[1:][0])
    if float > 0:
        return f'{cel}.{float}'
    return cel


def DecimalToStr(value):
    if value == None:
        return ''

    try:
        # if value == 0:
        #     return None

        return del_last_zero(value=value)
    except IndexError:
        return value


def StrToNumber(s1):
    if s1 == None:
        return None

    if not isinstance(s1, str):
        raise Exception(f'{s1} is not str type.')

    s = s1.replace(',', '.')

    if not isinstance(s, str):
        raise Exception(f'{s} is not str type.')
    try:
        return int(s)
    except ValueError:
        try:
            return float(s)
        except ValueError as ex:
            return None


def IntToNumber(s):
    if s == None:
        return None

    if not isinstance(s, int):
        raise Exception(f'{s} is not int type.')

    return float(s)


def IntToStr(value):
    if value == None:
        return None

    if not isinstance(value, str):
        return value

    if not isinstance(value, int):
        return del_last_zero(value=value)

    raise Exception(f'{value}  must be int')


def ToStr(s):
    if s == None or s == 'null':
        return None

    if not isinstance(s, str):
        return s

    if not isinstance(s, Decimal):
        return DecimalToStr(s)

    if not isinstance(s, int):
        return del_last_zero(value=s)

    return s


def ToNumber(s):
    if s == None:
        return 0

    if isinstance(s, int):
        return IntToNumber(s)
    elif isinstance(s, str):
        return StrToNumber(s)
    elif isinstance(s, float):
        return s
    elif isinstance(s, Decimal):
        return float(s)
    else:
        raise Exception(f'{s} unknown type.')


def ToInt(s):
    if s == None:
        return None

    if isinstance(s, int):
        return s
    elif isinstance(s, str):
        return StrToNumber(s)
    elif isinstance(s, float):
        return int(s)
    elif isinstance(s, Decimal):
        return int(s)
    else:
        raise Exception(f'{s} unknown type.')


def ToDecimal(s):
    if s == None:
        return 0

    if isinstance(s, int):
        return Decimal(s)
    elif isinstance(s, str):
        if len(s) == 0:
            return 0
        return Decimal(s)
    elif isinstance(s, float):
        return Decimal(s)
    elif isinstance(s, Decimal):
        return s
    else:
        raise Exception(f'{s} unknown type.')


def IntToDecimal(s):
    if s == None:
        return None

    if not isinstance(s, int):
        raise Exception(f'{s} is not int type.')

    return Decimal(s)


def StrToInt(s):
    if s == None:
        return None
    try:
        return int(s)
    except ValueError as ex:
        return None


def DelProps(value):
    if isinstance(value, dict):
        for key, _value in value.items():
            if isinstance(_value, BitHandler):
                setAttr(value, key, _value._value)
            elif isinstance(_value, Bit):
                setAttr(value, key, _value.is_set)
        return value
    else:
        value


def GetPropsInt(value):
    if isinstance(value, BitHandler):
        return value._value
    else:
        value


def model_2_dict(instance, fields=None, exclude=None):
    """
    Return a dict containing the data in ``instance`` suitable for passing as
    a Form's ``initial`` keyword argument.

    ``fields`` is an optional list of field names. If provided, return only the
    named.

    ``exclude`` is an optional list of field names. If provided, exclude the
    named from the returned dict, even if they are listed in the ``fields``
    argument.
    """
    opts = instance._meta
    data = {}
    for f in chain(opts.concrete_fields, opts.private_fields, opts.many_to_many):
        if not getattr(f, 'editable', False):
            continue
        if fields != None and f.name not in fields:
            continue
        if exclude and f.name in exclude:
            continue
        if isinstance(f, ForeignKey):
            data[f'{f.name}_id'] = f.value_from_object(instance)
        else:
            data[f.name] = f.value_from_object(instance)
    return data


def compare_2_dict(dict1, dict2):
    if not isinstance(dict1, dict):
        raise Exception(f'dict1 must be dict')

    if not isinstance(dict2, dict):
        raise Exception(f'dict2 must be dict')

    messages = []

    for x_values, y_values in zip(dict1.items(), dict2.items()):
        if x_values != y_values:
            messages.append(f'{x_values} != {y_values}')
    return messages


def flen(iterable):
    return reduce(lambda sum, element: sum + 1, iterable, 0)


class Set:
    _lst = []
    _set = set()

    def __init__(self, lst=set()):
        if isinstance(lst, set):
            self._set.clear()
            for l in list(lst):
                self._set.add(l)

        if isinstance(lst, list):
            self._lst = lst

    def __str__(self):
        if self._set != None:
            _lst = list(self._set)
            _lst = [str(l) for l in _lst]

            _str = ' , '.join(_lst)
            return f'''({_str})'''
        elif self._lst != None:
            _lst = [str(l) for l in self._lst]
            _str = ' , '.join(_lst)
            return f'''({_str})'''
        else:
            return '()'

    def is_exists(self, element):
        _len = len(self._set)
        self._set.add(element)
        return _len == len(self._set)

    @property
    def get_set_sorted_as_original(self):
        if self._lst != None:
            return sorted(set(self._lst), key=self._lst.index)
        elif self._set != None:
            return list(self._set)
        else:
            return []



# Generated by Django 2.2 on 2019-04-14 08:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0011_auto_20190414_0801'),
    ]

    operations = [
        migrations.AddField(
            model_name='messages',
            name='deleted_at',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления'),
        ),
        migrations.AddField(
            model_name='messages_state',
            name='deleted_at',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления'),
        ),
        migrations.AddField(
            model_name='messages_theme',
            name='deleted_at',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления'),
        ),
    ]



from django.urls import path

from clndr.views import calendars

urlpatterns = [

    path('Calendars/Fetch/', calendars.Calendars_Fetch),
    path('Calendars/Add', calendars.Calendars_Add),
    path('Calendars/Update', calendars.Calendars_Update),
    path('Calendars/Remove', calendars.Calendars_Remove),
    path('Calendars/Lookup/', calendars.Calendars_Lookup),
    path('Calendars/Info/', calendars.Calendars_Info),
    path('Calendars/Copy', calendars.Calendars_Copy),

]



import logging

from django.core.management import BaseCommand
from django.db import connections, connection, transaction

logger = logging.getLogger(__name__)


class Command(BaseCommand):

    def handle(self, *args, **options):
        with transaction.atomic():
            with connection.cursor() as cursor:
                cursor.execute('update twits_chat_user_user set common_id = 0')

            with connection.cursor() as cursor:
                cursor.execute('''select
                                        user_reciver_id
                                    from
                                        twits_chat_user_user
                                    group by
                                        user_reciver_id
                                    order by user_reciver_id''')
                user_recivers = cursor.fetchall()
                for user_reciver in user_recivers:
                    print(user_reciver[0])
                    user_reciver_id = user_reciver[0]
                    cursor.execute("select id, user_sender_id from twits_chat_user_user  where user_reciver_id = %s order by user_sender_id", [user_reciver[0]])
                    user_senders = cursor.fetchall()

                    for user_sender in user_senders:
                        print(f'   {user_sender}')

                        from twits.models.chat_user_user import Chat_user_userManager
                        common_id = Chat_user_userManager.get_next_common_id()
                        id = user_sender[0]
                        user_sender_id = user_sender[1]

                        cursor.execute("select id from twits_chat_user_user  where user_reciver_id = %s and user_sender_id=%s", [user_sender_id, user_reciver_id])
                        rev_user_senders = cursor.fetchall()

                        for rev_user_sender in rev_user_senders:
                            print(f'       {rev_user_sender}')
                            rev_id = rev_user_sender[0]

                            cursor.execute("update twits_chat_user_user set common_id = %s where id = %s", [common_id, rev_id])
                        cursor.execute("update twits_chat_user_user set common_id = %s where id = %s", [common_id, id])

                cursor.execute("delete from twits_chat_user_user_theme where chat_user_user_id in (select id from twits_chat_user_user where common_id = 0)")
                cursor.execute("delete from twits_chat_user_user where common_id = 0")

        print('Recreate_tetatet_theme done.')



from django.urls import path

from clndr.views import shift_day_relax

urlpatterns = [

    path('Shift_day_relax/Fetch/', shift_day_relax.Shift_day_relax_Fetch),
    path('Shift_day_relax/Add', shift_day_relax.Shift_day_relax_Add),
    path('Shift_day_relax/Update', shift_day_relax.Shift_day_relax_Update),
    path('Shift_day_relax/Remove', shift_day_relax.Shift_day_relax_Remove),
    path('Shift_day_relax/Lookup/', shift_day_relax.Shift_day_relax_Lookup),
    path('Shift_day_relax/Info/', shift_day_relax.Shift_day_relax_Info),
    path('Shift_day_relax/Copy', shift_day_relax.Shift_day_relax_Copy),

]



# Generated by Django 2.2.4 on 2019-08-22 22:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0026_auto_20190822_2217'),
    ]

    operations = [
        migrations.AlterField(
            model_name='shift_day',
            name='begindinnertime',
            field=models.TimeField(db_index=True, default=None, verbose_name='Начало перерыва на обед'),
        ),
        migrations.AlterField(
            model_name='shift_day',
            name='enddinnertime',
            field=models.TimeField(db_index=True, default=None, verbose_name='Конец перерыва на обед'),
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-26 00:25

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import isc_common.fields.code_field
import isc_common.fields.related


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Entity_1c',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('code', isc_common.fields.code_field.CodeField(unique=True)),
            ],
            options={
                'verbose_name': 'Сущность 1С',
            },
        ),
        migrations.CreateModel(
            name='Param_type',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('code', isc_common.fields.code_field.CodeField(unique=True)),
            ],
            options={
                'verbose_name': 'Типы параметров',
            },
        ),
        migrations.CreateModel(
            name='Document_1c',
            fields=[
                ('ref', models.UUIDField(primary_key=True, serialize=False)),
                ('entity', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='one_c.Entity_1c')),
            ],
            options={
                'verbose_name': 'Документ',
            },
        ),
        migrations.CreateModel(
            name='Documents_param_1c',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('value', models.TextField(db_index=True)),
                ('document', isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to='one_c.Document_1c')),
                ('parent', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='one_c.Documents_param_1c')),
                ('type', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='one_c.Param_type')),
            ],
            options={
                'verbose_name': 'Параметры документов 1С',
                # 'unique_together': {('document', 'type')},
            },
        ),
    ]



# Generated by Django 3.1 on 2020-08-08 04:28

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('history', '0013_auto_20200208_1133'),
    ]

    operations = [
        migrations.AlterField(
            model_name='history',
            name='ip_address',
            field=models.GenericIPAddressField(),
        ),
    ]



# Generated by Django 2.2.2 on 2019-06-11 10:02

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0026_auto_20190610_1436'),
    ]

    operations = [
        migrations.DeleteModel(
            name='History',
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-08 17:14

from django.db import migrations
import isc_common.fields.description_field


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0031_user_props'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='description',
            field=isc_common.fields.description_field.DescriptionField(),
        ),
    ]



# Generated by Django 2.2.3 on 2019-07-06 11:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('one_c', '0011_auto_20190706_1040'),
    ]

    operations = [
        migrations.AddField(
            model_name='documents_param_1c',
            name='value_boolean',
            field=models.BooleanField(blank=True, db_index=True, null=True),
        ),
        migrations.AddField(
            model_name='documents_param_1c',
            name='value_float',
            field=models.FloatField(blank=True, db_index=True, null=True),
        ),
        migrations.AddField(
            model_name='documents_param_1c',
            name='value_int',
            field=models.IntegerField(blank=True, db_index=True, null=True),
        ),
    ]
